"""
InnerContacts

author: jbonet
date:   02/2014

@oliva's lab
"""
from .   import PPInnerContact, PHInnerContact
from SBI import SBIglobals


class InnerContacts(object):
    def __init__(self, pdb, AA = False, AA_type = "cb",  AA_distance = 12,
                            NC = False, NC_type = "min", NC_distance = 8,
                            HT = True,  HT_type = "min", HT_distance = 6):
        self._pdb          = pdb

        self._AAdo        = AA
        self._AAcontacts  = {}
        self._AA_type     = AA_type
        self._AA_distance = AA_distance

        self._NCdo        = NC
        self._NCcontacts  = {}
        self._NC_type     = NC_type
        self._NC_distance = NC_distance

        self._HTdo        = HT
        self._HTcontacts  = {}
        self._HT_type     = HT_type
        self._HT_distance = HT_distance

        self._build()

    #
    # ATTRIBUTES
    #
    @property
    def pdb(self):        return self._pdb

    @property
    def AAcontacts(self): return self._AAcontacts.values()

    @property
    def NCcontacts(self): return self._NCcontacts.values()

    @property
    def HTcontacts(self): return self._HTcontacts.values()

    #
    # PRIVATE FUNCTIONS
    #
    def _build(self):
        count  = 1

        SBIglobals.alert('debug', self, 'Analyzing Inner Contacts of {0:03} chains'.format(len(self.pdb)))

        for chain in self.pdb.chains:
            SBIglobals.alert('debug', self, 'Analyzing Chain {0:03} out of {1:03}'.format(count, len(self.pdb)))
            self._add_AAc(chain)
            self._add_NCc(chain)
            self._add_HTc(chain)
            count += 1

    def _add_AAc(self, chain):
        if self._AAdo and chain.chaintype == 'P':
            raise NotImplementedError
            ppi_id = PPInnerContact.test_identifier(chain)
            SBIglobals.alert('debug', self, 'Analyzing Protein Inner Contacts {0} for {1.chain}'.format(ppi_id, chain))

            ppi = PPInnerContact(chain, self._AA_type, self._AA_distance)

            SBIglobals.alert('debug', self, '\tAdding new Inner Contacts')
            self._AAcontacts[ppi_id] = ppi

    def _add_NCc(self, chain):
        if self._NCdo and chain.chaintype == 'N':
            raise NotImplementedError

    def _add_HTc(self, chain):
        if self._HTdo and chain.chaintype == 'P':
            phi_id = PHInnerContact.test_identifier(chain)
            SBIglobals.alert('debug', self, 'Analyzing Protein-Heteroatom Inner Contacts {0} for {1.chain}'.format(phi_id, chain))

            phi = PHInnerContact(chain, self._HT_type, self._HT_distance)

            SBIglobals.alert('debug', self, '\tAdding new Inner Contacts')
            self._HTcontacts[phi_id] = phi

    #
    # OVERRIDE DEFAULT METHODS
    #
    def __str__(self):
        data = []
        data.append("Inner Contacts for {0}".format(self.pdb.id))
        for ppi in self._AAcontacts:
            if len(self._AAcontacts[ppi]) > 0:
                data.append(self._AAcontacts[ppi].toString(True))
        for pni in self._NCcontacts:
            if len(self._NCcontacts[pni]) > 0:
                data.append(self._NCcontacts[pni].toString(True))
        for phi in self._HTcontacts:
            if len(self._HTcontacts[phi]) > 0:
                data.append(self._HTcontacts[phi].toString(True))
        return "\n".join(data)



from SBI.data                  import Element, element_dic
from SBI.databases             import PDBeChem

tables = {'hetero'  : 'hetero', 
          'hparent' : 'hetero_parent', 
          'formula' : 'hetero_formula',
          'atom'    : 'atom'}

class Element(Element):
    def __init__(self, number, symbol, name):
        super(Element, self).__init__(number, symbol, name)

    def toSQL(self):
        return 'INSERT INTO {0} VALUES ("{1.symbol}", "{1.name}", {1.number});'.format(tables['atom'], self)

class PDBeChem(PDBeChem):
    """
    """
    def __init__(self, cif_file):
        super(PDBeChem, self).__init__(cif_file)

    def toSQL(self):
        data = []
        self._name = self.name.replace('"','\\"')
        values  = '("{0.id}","{0.name}","{0.formula}",{0.weight},{0.formal_charge})'.format(self)
        data.append('INSERT INTO {0} VALUES {1};'.format(tables['hetero'], values))
        if self.parent is not None:
            for p in self.parent:
                if p != "":
                    data.append('INSERT INTO {0} VALUES ("{1}","{2}");'.format(tables['hparent'], self.id, p))
        for atom in self.full_formula:
            data.append('INSERT INTO {0} VALUES ("{1.id}","{2}",{3});'.format(tables['formula'], self, atom, self.full_formula[atom]))

        return "\n".join(data)


from BioLib.Algebra.Transforms import get_points_distance, hex_transform_point, pachdock_transform_point
from BioLib.Tools.BioExceptions import RelocateProgramError

class Atom(object):
    '''
    Represents an structure atom
    '''
    def __init__(self, num, atom_type, x, y, z):
        '''
        Constructor
        '''
        self.num = num
        self.type = atom_type
        self.x = x
        self.y = y
        self.z = z

    def get_num(self):
        return self.num

    def get_type(self):
        return self.type

    def get_x(self):
        return self.x

    def get_y(self):
        return self.y

    def get_z(self):
        return self.z

    def get_coords(self):
        return (self.x, self.y, self.z)

    def get_distance(self, atom):
        return get_points_distance(self.get_coords(), atom.get_coords())
 
    def get_distance_coords(self, coords):
        return get_points_distance(self.get_coords(), coords)

    def relocate(self, translation_vector, rotation_vector, docking='HEX'):
        '''
        Relocate the atom position (x, y, z). 
        @translation_vector = Vector used to translate the atom
        @Rotation_vector = Vector used to rotate the structure
        @docking = Docking program used to get the vectors
        '''
        if docking == 'HEX':
            self.x, self.y, self.z = hex_transform_point(self.get_coords(), translation_vector, rotation_vector)
        elif docking == 'PATCHDOCK':
            self.x, self.y, self.z = pachdock_transform_point(self.get_coords(), translation_vector, rotation_vector)
        else:
            raise RelocateProgramError(docking)

    def __str__(self):
        '''
        Return the atom information
        '''
        return 'ATOM%s  %s%s%s%s' % (str(self.get_num()).rjust(7), 
                                     self.get_type().ljust(3), 
                                     ('%.3f' % float(self.get_x())).rjust(12), 
                                     ('%.3f' % float(self.get_y())).rjust(8), 
                                     ('%.3f' % float(self.get_z())).rjust(8))



import os,sys,re
import argparse
import ConfigParser
import shutil
import string
import numpy as np
import scipy.stats as st
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt

def fileExist (file):
 if file is not None:
  return os.path.exists(file) and os.path.isfile(file)
 else:
  return False

def reduce_outliers_lstsq(y,x,m,c,t):
    yy=[]
    xx=[]
    d={}
    n = int(len(x)*t/100)
    out=0
    for i in xrange(len(x)):
        b = m*x[i] + c
        d.setdefault(i,(abs(y[i] - b)))
    for key, value in sorted(d.iteritems(), key=lambda (k,v): (v,k),reverse=True):
        out = out + 1
        if out > n:
           yy.append(y[key])
           xx.append(x[key])
        else:
           sys.stdout.write("Outlier: %10d \tY= %f \tX= %f \tRMSD= %f\n"%(key,y[key],x[key],value))
    return (yy,xx)

def parse_user_arguments(*args, **kwds):

    parser = argparse.ArgumentParser(
        description = "Parse SKEMPI and create MODPIN files",
        epilog      = "@oliva's lab 2018")
    parser.add_argument('-a','--ddG',dest='ddg',action = 'store',default=None,
                        help = 'SKEMPI parsed file of affinities')
    parser.add_argument('-b','--FEP',dest='fep',action = 'store',default=None,
                        help = 'Free Energy Perturbation file obtained with MODPPI models')
    parser.add_argument('-lt','--outliers_threshold',dest='outlier_threshold',action = 'store',default=None,
                        help = 'Free Energy Perturbation Outliers energy (limits max and min of FEP)')
    parser.add_argument('-lp','--outliers_deviation',dest='outlier_percentil',action = 'store',default='0.0',
                        help = 'Percentage of Outliers (neglects a percentage of data, default is 0 and uses all)')
    parser.add_argument('-u','--use_score',dest='score',action='store_true',
                        help = 'Flag to use average difference of scores instead of FEP')
    parser.add_argument('-z','--use_partition_function',dest='partition',action='store_true',
                        help = 'Flag to use ratio of partition functions: -kT log(Zmut/Zwt) instead of FEP')
    parser.add_argument('-s','--show_canvas',dest='canvas',action = 'store_true',
                        help = 'Show canvas figure after execution (not shown by default)')
    parser.add_argument('-o','--output_file',dest='out',action = 'store', default='output',
                        help = 'Output rootname for files (default is output)')
    parser.add_argument('-f','--format_image',dest='format',action = 'store',default='png',
                        help = 'Format of the image in the output file: png(default), pdf, ps, eps and svg ')
    parser.add_argument('-k','--factor_lambda',dest='factor_lambda',action = 'store',default='1.0',
                        help = 'Factor factor_lambda to multiply the energies (default is 1) ')
    parser.add_argument('-v','--verbose',dest='verbose',action = 'store_true',
                        help = 'Verbose execution ')
    parser.add_argument('-c','--clean_null',dest='clean_null',action = 'store',default=None,
                        help = 'Clean ddG values near 0.0 because they are caused by errors (default is None, otherwise select a threshold near 0, i.e. 0.1)')




    options=parser.parse_args()

    return options

  

def main():

 options   = parse_user_arguments()

 if options.ddg is None or options.fep is None  :
    sys.stderr.write("Missing arguments, Check help\n")
    exit(0)

 if options.verbose: sys.stdout.write("Parsing %s\n"%(options.ddg))
 ddg_dict={}
 if not fileExist(options.ddg):
    sys.stderr.write("File %s not found\n"%options.ddg)
 fa=open(options.ddg,"r")
 for line in fa:
    if line.startswith("#"): continue
    form,ddg=line.split()
    if options.clean_null is not None:
      if ddg < -float(options.clean_null) or ddg > float(options.clean_null):
        ddg_dict.setdefault(form,float(ddg))
    else:
      ddg_dict.setdefault(form,float(ddg))
 fa.close()


 if options.verbose: sys.stdout.write("Parsing %s\n"%(options.fep))
 fep_dict={}
 if not fileExist(options.fep):
    sys.stderr.write("File %s not found\n"%options.fep)
 fb=open(options.fep,"r")
 for line in fb:
    data=line.split()
    if data[0].startswith("#"): continue
    data=line.split()
    form=data[-1]
    if options.score:
       if data[-2] is not "nan" and  data[-2] is not "inf": fep=float(data[6])*float(options.factor_lambda)
    elif options.partition:
       if data[4] is not "nan" and  data[4] is not "inf": fep=float(data[4])*float(options.factor_lambda)
    else:
       if data[3] is not "nan" and  data[3] is not "inf": fep=float(data[3])*float(options.factor_lambda)
    if options.outlier_threshold is not None:
     if options.clean_null is not None:
      if fep < -float(options.clean_null) or fep > float(options.clean_null):
        if fep < float(options.outlier_threshold) and fep > -float(options.outlier_threshold): fep_dict.setdefault(form,fep)
     else:
      if fep < float(options.outlier_threshold) and fep > -float(options.outlier_threshold): fep_dict.setdefault(form,fep)
    else:
     if options.clean_null is not None:
      if fep < -float(options.clean_null) or fep > float(options.clean_null):
        fep_dict.setdefault(form,fep)
     else:
      fep_dict.setdefault(form,fep)
      
 fb.close()


 ddg_list=[]
 fep_list=[]
 for form,ene in ddg_dict.iteritems():
   if fep_dict.has_key(form):
      ddg_list.append(ene)
      fep_list.append(fep_dict.get(form))
      


 
 ddg_real=np.array(ddg_list)
 ddg_pred=np.array(fep_list)


 (pearson,p_value)=st.pearsonr(ddg_real,ddg_pred)

 A = np.vstack([ddg_pred, np.ones(len(ddg_pred))]).T
 m, c = np.linalg.lstsq(A, ddg_real,rcond=-1 )[0]

 output= options.out+".dat"
 graph = options.out+"."+options.format

 if float(options.outlier_percentil) > 0:
   (ddg_list_reduced,fep_list_reduced)=reduce_outliers_lstsq(ddg_list,fep_list,m,c,float(options.outlier_percentil))
   ddg_real=np.array(ddg_list_reduced)
   ddg_pred=np.array(fep_list_reduced)
   (pearson,p_value)=st.pearsonr(ddg_real,ddg_pred)
   A = np.vstack([ddg_pred, np.ones(len(ddg_pred))]).T
   m, c = np.linalg.lstsq(A, ddg_real,rcond=-1 )[0]


 fo=open(output,"w")
 fo.write("Pearson Correlation:\t%f\n"%pearson)
 fo.write("Two-tailed p-value: \t%f\n"%p_value)
 fo.write("Slope of fitting:   \t%f\n"%m)
 fo.write("Cut Y-axis:         \t%f\n"%c)
 fo.close()

 image, fig = plt.subplots()

 fig.plot(ddg_pred, ddg_real, 'o', label='SKEMPI ddG', markersize=2)
 fig.plot(ddg_pred, m*ddg_pred + c, 'r', label='Fitted line')
 fig.legend()

 fig.set(xlabel='ddG predicted (Kcal/mol)', ylabel='ddG experimental (Kcal/mol)',
       title='Compare ddG of SKEMPI and ddG predicted by MODPIN')
 image.savefig( graph)
 if options.canvas:
    plt.show()



if __name__=="__main__":
 main()

 




import re

tables = {'scop'     : 'scop',
          'main'     : 'PDB',
          'scop_d'   : 'scop_description',
          'scop_pdb' : 'chain2scop',
          'chain'    : 'chain',
          'oldPDB'   : 'oldPDB'}

class SCOP(object):
    accepted = ['cl','cf','sf','fa','dm']
    def __init__(self):
        self._descriptions = []
        self._relations    = []
        self._norepel      = set()
        self._maps         = []

    def add_description(self, line):
        line = line.strip().split()
        # if line[1] in self.accepted and line[2] != 'unassigned-sccs':
        if line[1] in self.accepted:
            if line[1] == 'cl': line[-1] = ''
            self._descriptions.append([line[0], line[1], line[2], " ".join(line[4:]).replace("'","\\'")])

    def add_relation(self, line):
        line = line.strip().split()
        # if line[0] == 'unassigned-sccs':return

        data = tuple([tuple(x.split('=')) for x in line[5].split(',')][0:5])

        if not data[4][1] in self._norepel:
            self._relations.append(data)
            self._norepel.add(data[4][1])

        info = []
        if ',' in line[2]:
            multi = line[2].split(',')
            for m in multi:
                info.append(list(line))
                info[-1][2] = m
        else:
            info.append(line)
        for l in info:
            assignation = [l[1],
                           'A' if l[2] == '-' else l[2] if not ':' in l[2] else l[2].split(':')[0],
                           '' if (l[2] == '-' or not ':' in l[2]) else l[2].split(':')[1],
                           data[4][1]]

            self._maps.append(assignation)

    @staticmethod
    def prepdbdeleted():
        command = "INSERT INTO {0} (pdb,date,header,method) VALUES ('0000','000-00-00','FAKE_PROTEIN','X-RAY DIFFRACTION');\n".format(tables['main'])
        chains = "ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890abcdefghijklmnopqrstuvwxyz"
        for c in chains:
            command += "INSERT INTO {0} (pdb,chain,start,end) VALUES ('0000','{1}',1,2);\n".format(tables['chain'], c)
        return command
    @staticmethod
    def afterpdbdeleted():
        command = "DELETE FROM {0} WHERE chain IN (SELECT nid FROM {1} WHERE pdb='0000');\n".format(tables['scop_pdb'], tables['chain']) 
        command += "DELETE FROM {0} WHERE pdb = '0000';\n".format(tables['chain'])
        command += "DELETE FROM {0} WHERE pdb = '0000';\n".format(tables['main'])
        return command
    def toSQL(self):
        command = ''
        for d in self._descriptions:
            command += "INSERT INTO {0} (id,type,code,description) VALUES ({1[0]},'{1[1]}','{1[2]}','{1[3]}');\n".format(tables['scop_d'], d)
        for r in self._relations:
            command += "INSERT INTO {0} (domain,family,superfamily,fold,class) VALUES ({1[4][1]},{1[3][1]},{1[2][1]},{1[1][1]},{1[0][1]});\n".format(tables['scop'], r)
        for a in self._maps:
            command += "SET @nid = NULL;\n";
            command += "SELECT nid, start, idxs, end, idxe INTO @nid, @p1, @i1, @p2, @i2 FROM {0} WHERE pdb=COALESCE((SELECT pdb from {3} WHERE oldid = '{1}'),'{1}') AND chain='{2}';\n".format(tables['chain'],a[0].upper(),a[1], tables['oldPDB'])
            command += "SET @nid = (SELECT IF (@nid IS NULL, (SELECT nid FROM {0} WHERE pdb='0000' AND chain='A'), @nid));\n".format(tables['chain'])
            if a[2] != '':
                m = re.search('(-*\d+)(\w*)-{1}(-*\d+)(\w*)',a[2])
                command += "SET @p1 = {0};\n".format(  m.group(1))
                command += "SET @i1 = '{0}';\n".format(' ' if m.group(2) == '' else m.group(2))
                command += "SET @p2 = {0};\n".format(  m.group(3))
                command += "SET @i2 = '{0}';\n".format(' ' if m.group(4) == '' else m.group(4))
            command += "INSERT INTO {0} (domain,start,idxs,end,idxe,chain) VALUES ({1},@p1,@i1,@p2,@i2,@nid);\n".format(tables['scop_pdb'],a[3])
        return command


import re, copy
import BioLib.Structure.PDB as PDB

class HEXDock():
    '''
    Class to manage the HEX Docking transform file
    '''
    def __init__(self, HEXdockFile):
        '''
        Call the parsing method
        '''
        self.decoys = []
        self.__parse(HEXdockFile)

    def __parse(self, HEXdockFile):
        '''
        '''
        hexdockfo = open(HEXdockFile, 'r')
        for line in hexdockfo:
            if line[0] != '#' and len(line)>1:
                decoy = re.split('\s+', line.strip('\n'))
                self.decoys.append(HEXDecoy(int(decoy[0]), int(decoy[1]), float(decoy[2]), float(decoy[3]), float(decoy[4]), float(decoy[5]), float(decoy[6]), float(decoy[7]), float(decoy[8]), float(decoy[9])))
        hexdockfo.close()
        
    # GETTERS #

    def get_decoys(self):
        return self.decoys

    def get_decoy_by_id(self, cluster):
        for decoy in self.decoys:
            if decoy.get_cluster() == cluster:
                return decoy
        return None

    # METHODS #

    def get_num_decoys(self):
        return len(self.get_decoys())

    def get_structures(self, mobile_structure):
        '''
        ATTENTION: Memory inefficient
        '''
        structure_list = []
        for decoy in self.get_decoys():
            structure_list.append(decoy.get_structure(mobile_structure))
        return structure_list

    def filter_by_spatial_restrictions(self, static_structure, mobile_structure, spatial_restrictions):
        '''
        Filter the decoys by spatial restrictions
        '''
        filtered_decoys = []
        for decoy in self.get_decoys():
            compatible = True
            for spatial_restriction in spatial_restrictions:
                decoy_structure = decoy.get_structure(mobile_structure)
                distance = static_structure.get_residue_by_num(spatial_restriction[0]).get_residue_distance(decoy_structure.get_residue_by_num(spatial_restriction[1]), 'ca')
                if spatial_restriction[3] == 'max':
                    if distance > spatial_restriction[2]:
                        compatible = False
                if spatial_restriction[3] == 'min':
                    if distance < spatial_restriction[2]:
                        compatible = False
            if compatible:
                filtered_decoys.append(decoy)
                print "decoy %d appended" % decoy.get_cluster()
            else:
                print "decoy %d skipped" % decoy.get_cluster()
        self.decoys = filtered_decoys

    def print_structures(self, static_structure, mobile_structure, pdbFile, singlePDB=False):
        '''
        Print all PDB decoys
        '''
        decoyList = []
        for decoy in self.get_decoys():
            if singlePDB:
                decoy_structure = decoy.get_structure(mobile_structure)
                decoyList.append((static_structure, decoy_structure))
            else:
                decoy.print_structure(static_structure, decoy_structure, pdbFile+'_'+str(decoy.get_cluster())+'.pdb')
        if singlePDB:
            PDB.write_pdb(decoyList, pdbFile+'.pdb', multi_chain=True, multi_model=True)

    def __iter__(self):
        '''
        Iterates over each decoy
        '''
        for decoy in self.decoys:
            yield decoy

    def __str__(self):
        '''
        toString method
        '''
        return '\n'.join([decoy.__str__() for decoy in self.decoys])

class HEXDecoy():
    '''
    Object that stores a decoy result from an HEX docking
    '''

    def __init__(self, cluster, solution, RMS, energy, x, y, z, alpha, beta, gamma):
        '''
        '''
        self.cluster = cluster
        self.solution = solution
        self.RMS = RMS
        self.energy = energy
        self.x = x
        self.y = y
        self.z = z
        self.alpha = alpha
        self.beta = beta
        self.gamma = gamma

    # GETTERS #

    def get_cluster(self):
        return self.cluster

    def get_solution(self):
        return self.solution

    def get_RMS(self):
        return self.RMS

    def get_energy(self):
        return self.energy

    def get_tranlation_vector(self):
        return (self.x, self.y, self.z)
   
    def get_rotation_vector(self):
        return (self.alpha, self.beta, self.gamma)

    def get_structure(self, mobile_structure):
        '''
        Gets the structure of the given docking result
        '''
        structure = copy.deepcopy(mobile_structure)
        structure.relocate(self.get_tranlation_vector(), self.get_rotation_vector()) 
        return structure

    def print_structure(self, static_structure, mobile_structure, pdbFile):
        decoy_structure = self.get_structure(mobile_structure)
        PDB.write_pdb((static_structure, decoy_structure), pdbFile+'.pdb', multi_chain=True, multi_model=False)

    def __str__(self):
        return '%d %d %f %f %f %f %f %f %f %f' % (self.cluster, self.solution, self.RMS, self.energy, self.x, self.y, self.z, self.alpha, self.beta, self.gamma)



'''
@file: CDhitExe.py

@author: Jaume Bonet
@mail:   jaume.bonet@gmail.com
@date:   2013

@ [oliva's lab](http://sbi.imim.es)

@class: CDhitExe
'''
import os

from SBI.external       import ExternalExe
from SBI.external.CDhit import CDhitList  as CHL
from SBI.sequence       import Fasta
from SBI.beans          import Executable
from SBI                import SBIglobals as SBIg


class CDhitExe(ExternalExe):
    '''
    Executes CD-hit program over a FASTA file.

    '''
    def __init__(self, fasta, threshold, step = None, memory = None,
                 output_dir = None, execute = True):
        '''
        @param:    fasta
        @pdef:     input FASTA file
        @ptype:    {String}

        @param:    threshold
        @pdef:     maximum allowed homology. Between 0 and 1
        @ptype:    {Float}

        @param:    step
        @pdef:     execute a partial clustering to speed the process.
                   Between 0 and 1
        @ptype:    {Float}

        @param:    memory
        @pdef:     maximum memory allowed to use. default is 400 Mb
        @ptype:    {Integer}

        @param:    output_dir
        @pdef:     directory to print the output to
        @ptype:    {String}

        @param:    execute
        @pdef:     flag. automatically execute when creating the object.
        @pdefault: _True_
        @ptype:    {Boolean}
        '''

        fasta = os.path.abspath(fasta)

        if CDhitExe._EXE is None:
            self._set_default_executable('cd-hit')

        self._input     = fasta
        self._outdir    = os.path.split(fasta)[0] if output_dir is None else output_dir
        self._output    = self._build_output_file(fasta, threshold)
        self._threshold = str(threshold)
        self._word      = self._set_word_size(threshold)
        self._memory    = memory
        self._step      = step
        if step is not None:
            self._step_file = self._build_output_file(fasta, step)

        if execute:
            self._execute()

        self._cdhitlist = None

    ##############
    # ATTRIBUTES #
    ##############
    @property
    def output_file(self):
        '''
        Name of the cd-hit output file.

        @return: {String}
        '''
        return self._output + '.clstr'

    @property
    def master_fasta_file(self):
        '''
        Name of the fasta output file.

        @return: {String}
        '''
        return self._output + '.fa'

    ###########
    # METHODS #
    ###########
    def compare(self, fasta_file, output_file):
        '''
        Using the same threshold as before, creates a list of those proteins in
        fasta_file similar to the first input and a fasta file with the
        remaining proteins.

        @param:    fasta_file
        @pdef:     name of the Fasta file
        @ptype:    {String}

        @param:    output_file
        @pdef:     prefix of the output file
        @ptype:    {String}

        @return: {List} of {String}: list file name, fasta file name
        '''
        e = Executable(executable = self._EXE.executable + '-2d',
                       path       = self._EXE.path)
        e.add_attribute(self._input,      '-i')
        e.add_attribute(fasta_file,       '-i2')
        e.add_attribute(output_file,      '-o')
        e.add_attribute(self._threshold,  '-c')
        e.add_attribute('1',              '-g')
        e.add_attribute(self._word,       '-n')
        if self._memory is not None:
            e.add_attribute(self._memory, '-M')
        try:
            e.execute()
            e.clean_command()
        except SystemError, er:
            SBIg.throw(self, 'Some error occurred while executing cd-hit\n{0}\n'.format(er),
                       SystemError)

    def make_master_fasta(self, output_fasta = None, force = None):
        '''
        Directly create a fasta file from the master sequences of each
        cluster of a list.

        @param:    output_fasta
        @pdef:     name of the Fasta file
        @pdefault: input fasta + threshold + '.fa'
        @ptype:    {String}

        @param:    force
        @pdef:     overwrite previous files with the same name
        @pdefault: _SBIglobals.overwrite_
        @ptype:    {Boolean}

        @return: {Fasta}
        '''
        if output_fasta is None:
            output_fasta = self.master_fasta_file
        clist = self.parse()
        prots = set()
        fs    = Fasta(self._input)
        for c in clist.clusters:
            prots.add(c.master.name)
        seq = fs.retrieve(prots)
        return Fasta.build_multifasta(output_fasta, seq, force = force)

    def parse(self):
        '''
        Read the cd-hit output automatically generated by the execution.

        @return: {CDhitList}
        '''
        if self._cdhitlist is not None:
            return self._cdhitlist

        if self._step is None:
            self._cdhitlist = self.parse_cdhit_output(self.output_file)
        else:
            clist = self.parse_cdhit_output(self._step_file + '.clstr')
            clist.merge_clusters(self.output_file)
            self._cdhitlist = clist
        return self._cdhitlist

    @staticmethod
    def parse_cdhit_output(cdhit_file):
        '''
        Parse a given cd-hit output.

        @param:    cdhit_file
        @pdef:     name of the cd-hit output file
        @ptype:    {String}
        '''
        return CHL.CDhitList(cdhit_file)

    @staticmethod
    def dynamic(executable, path):
        '''
        Manually set the values for cd-hit path and executable .

        @param:    executable
        @pdef:     name of the executable file
        @ptype:    {String}

        @param:    path
        @pdef:     path to the executable file
        @ptype:    {String}
        '''
        CDhitExe._set_dynamic_executable(executable, path)

    ###################
    # PRIVATE METHODS #
    ###################
    def _build_output_file(self, infile, threshold):
        output_file = os.path.split(infile)[1] + '.' + str(threshold).replace('.', '_')
        return os.path.join(self._outdir, output_file)

    def _set_word_size(self, threshold):
        threshold = float(threshold)
        if threshold >= 0.7:
            word = 5
        elif threshold >= 0.6:
            word = 4
        elif threshold >= 0.5:
            word = 3
        elif threshold >= 0.4:
            word = 2
        else:
            word = 1
        return word

    def _execute(self):
        if self._step is None:
            self._simple_execute(self._threshold)
        else:
            self._word   = self._set_word_size(self._step)
            self._output = self._step_file
            self._simple_execute(self._step)
            infile       = self._input
            self._input  = self.output_file
            self._word   = self._set_word_size(self._threshold)
            self._output = self._build_output_file(infile, self._threshold)
            self._simple_execute(self._threshold)
            self._input  = infile

    def _simple_execute(self, threshold):
        self._EXE.add_attribute(self._input,     '-i')
        self._EXE.add_attribute(self._output,    '-o')
        SBIg.alert('debug', self,
                   'Setting threshold to {0}'.format(threshold))
        self._EXE.add_attribute(threshold,       '-c')
        self._EXE.add_attribute('1',             '-g')
        SBIg.alert('debug', self, 'Setting word to {0}'.format(self._word))
        self._EXE.add_attribute(self._word,      '-n')
        if self._memory is not None:
            self._EXE.add_attribute(self._memory, '-M')

        try:
            self._EXE.execute()
            self._EXE.clean_command()
        except SystemError, e:
            SBIg.throw(self, 'Some error occurred while executing cd-hit\n{0}\n'.format(e),
                       SystemError)



"""
Contact

author: jbonet
date:   03/2013

@oliva's lab
"""
import hashlib


class Contact(object):

    available_distance_types = set()
    atomic_types             = set()
    description              = ''

    def __init__(self, residue1, residue2):

        self._residue1  = residue1
        r1              = residue1.type + residue1.identifier
        self._residue2  = residue2
        r2              = residue2.type + residue2.identifier
        #self._md5       = tuple(sorted([hashlib.sha224(r1 + r2).hexdigest(),
        #                                hashlib.sha224(r2 + r1).hexdigest()]))
        self._md5       = tuple(sorted([hashlib.sha224(r1 + r2).hexdigest()]))
        self._distance  = {}

        self._underthreshold = False

        self._build()

    #
    # ATTRIBUTES
    #
    @property
    def md5s(self):
        return self._md5

    @property
    def description(self):
        return self.description

    #
    # BOOLEAN
    #
    @property
    def is_underthreshold(self):
        return self._underthreshold

    #
    # METHODS
    #
    def reverse(self):
        (self._residue1, self._residue2) = (self._residue2, self._residue1)
        for dist_type in self.available_distance_types:
            self._distance[dist_type] = (self._distance[dist_type][1], self._distance[dist_type][0], self._distance[dist_type][2])

    #
    # PRIVATE METHODS
    #
    def _build(self):
        raise NotImplementedError

    def _check_type(self, requested_type):
        if requested_type not in self.available_distance_types:
            raise AttributeError("Available distance types are {0}.\n".format(repr(self.available_distance_types)))

    #
    # OVERRIDE DEFAULT METHODS
    #
    def __eq__(self, other): return self.md5s == other.md5s
    def __ne__(self, other): return not self.__eq__(other)
    def __hash__(self):      return self.md5s.__hash__()

# class Contact(object):
#     """
#     Stores the basic information of a contact between two residues
#     """
#     def __init__(self, residue1, residue2, atom1 = None, atom2 = None, distance = 0, altdistance1 = -1, altdistance2 = -1):

#         self._residue1  = residue1
#         self._atom1     = atom1
#         self._residue2  = residue2
#         self._atom2     = atom2
#         self._distance  = [float(distance), float(altdistance1), float(altdistance2)]

#     @property
#     def residue1(self):
#         """
#         Data referring to residue1 of the contact
#         @rtype: {Residue}
#         """
#         return self._residue1

#     @property
#     def atom1(self):
#         """
#         Data referring to the atom of residue1 with the assigned interaction
#         @rtype: {Atom}
#         """
#         return self._atom1

#     @property
#     def residue2(self):
#         """
#         Data referring to residue2 of the contact
#         @rtype: {Residue}
#         """
#         return self._residue2

#     @property
#     def atom2(self):
#         """
#         Data referring to the atom of residue2 with the assigned interaction
#         @rtype: {Atom}
#         """
#         return self._atom2

#     @property
#     def distance(self):
#         """
#         Distance assigned to the contact
#         @rtype: Float
#         """
#         return self._distance[0]

#     #
#     # BOOLEAN
#     #
#     def compare_residues(self, contact):
#         """
#         Compares if the residue positions are the same

#         @type contact: {Contact}
#         @rtype: Boolean
#         """
#         if (self.residue1.number == contact.residue1.number) and \
#            (self.residue2.number == contact.residue2.number):
#             return True
#         return False

#     #
#     # FULL CONTACT DATA
#     #
#     def print_aldist(self):
#         if self._distance[1] == -1:
#             return str(self)
#         elif self._distance[2] == -1:
#             return '{0.residue1.type}:{0.residue1.number}\t{0.residue2.type}:{0.residue2.number}\t{0.distance}\t{0._distance[1]}'.format(self)
#         else:
#             return '{0.residue1.type}:{0.residue1.number}\t{0.residue2.type}:{0.residue2.number}\t{0.distance}\t{0._distance[1]}\t{0._distance[2]}'.format(self)

#     #
#     # OVERRIDE DEFAULT METHODS
#     #
#     def __repr__(self):
#         return '<{0.__class__.__name__}: [{0.residue1.type}, {0.residue1.number}, {0.atom1.name}] [{0.residue2.type}, {0.residue2.number}, {0.atom2.name}] {0.distance}>'.format(self)

#     def __str__(self):
#         return '{0.residue1.type}:{0.residue1.number}:{0.atom1.name}\t{0.residue2.type}:{0.residue2.number}:{0.atom2.name}\t{0.distance}'.format(self)



'''
#
# PROTEIN
#
'''

'''
CODING TRANSFORMATION
http://www.ebi.ac.uk/pdbe-srv/pdbechem/chemicalCompound/show/
'''

aminoacids3to1 = {
    'ALA': 'A', 'AZT': 'A', 'CHA': 'A', 'HPH': 'A', 'NAL': 'A', 'AIB': 'A', 'BAL': 'A',
    'DHA': 'A', 'BB9': 'A', 'ALM': 'A', 'AYA': 'A', 'BNN': 'A', 'CHG': 'A', 'CSD': 'A',
    'DAL': 'A', 'DNP': 'A', 'FLA': 'A', 'HAC': 'A', 'MAA': 'A', 'PRR': 'A', 'TIH': 'A',
    'TPQ': 'A', 'BB9': 'A',
    'ARG': 'R', 'ORN': 'R', 'ACL': 'R', 'ARM': 'R', 'AGM': 'R', 'HAR': 'R', 'HMR': 'R',
    'DAR': 'R',
    'ASN': 'N', 'MEN': 'N',
    'ASP': 'D', 'ASZ': 'D', '2AS': 'D', 'ASA': 'D', 'ASB': 'D', 'ASK': 'D', 'ASL': 'D',
    'ASQ': 'D', 'BHD': 'D', 'DAS': 'D', 'DSP': 'D',
    'ASX': 'B',
    'CYS': 'C', 'CYD': 'C', 'CYO': 'C', 'HCY': 'C', 'CSX': 'C', 'SMC': 'C', 'BCS': 'C',
    'BUC': 'C', 'C5C': 'C', 'C6C': 'C', 'CCS': 'C', 'CEA': 'C', 'CME': 'C', 'CSO': 'C',
    'CSP': 'C', 'CSS': 'C', 'CSW': 'C', 'CY1': 'C', 'CY3': 'C', 'CYG': 'C', 'CYM': 'C',
    'CYQ': 'C', 'DCY': 'C', 'OCS': 'C', 'SOC': 'C', 'EFC': 'C', 'PR3': 'C', 'SCH': 'C',
    'SCS': 'C', 'SCY': 'C', 'SHC': 'C', 'PEC': 'C',
    'GLN': 'Q', 'DGN': 'Q',
    'GLU': 'E', 'GLA': 'E', 'GLZ': 'E', 'PCA': 'E', '5HP': 'E', 'CGU': 'E', 'DGL': 'E',
    'GGL': 'E', 'GMA': 'E',
    'GLX': 'Z',
    'GLY': 'G', 'GL3': 'G', 'GLZ': 'G', 'GSC': 'G', 'SAR': 'G', 'MPQ': 'G', 'NMC': 'G',
    'MSA': 'G', 'DBU': 'G',
    'HIS': 'H', 'HSE': 'H', 'HSD': 'H', 'HI0': 'H', 'HIP': 'H', 'HID': 'H', 'HIE': 'H',
    '3AH': 'H', 'MHS': 'H', 'DHI': 'H', 'HIC': 'H', 'NEP': 'H', 'NEM': 'H',
    'ILE': 'I', 'IIL': 'I', 'DIL': 'I',
    'LEU': 'L', 'NLE': 'L', 'LOV': 'L', 'NLN': 'L', 'NLP': 'L', 'MLE': 'L', 'BUG': 'L',
    'CLE': 'L', 'DLE': 'L', 'MLU': 'L',
    'LYS': 'K', 'LYZ': 'K', 'ALY': 'K', 'TRG': 'K', 'SHR': 'K', 'LYM': 'K', 'LLY': 'K',
    'KCX': 'K', 'LLP': 'K', 'DLY': 'K', 'DM0': 'K',
    'MET': 'M', 'MSE': 'M', 'CXM': 'M', 'FME': 'M', 'OMT': 'M',
    'PHE': 'F', 'DAH': 'F', 'HPQ': 'F', 'DPN': 'F', 'PHI': 'F', 'PHL': 'F',
    'PRO': 'P', 'HYP': 'P', 'DPR': 'P', 'ECQ': 'P', 'POM': 'P', 'H5M': 'P',
    'SER': 'S', 'HSE': 'S', 'STA': 'S', 'SVA': 'S', 'SAC': 'S', 'SEL': 'S', 'SEP': 'S',
    'SET': 'S', 'OAS': 'S', 'DSN': 'S', 'MIS': 'S',
    'THR': 'T', 'PTH': 'T', 'ALO': 'T', 'TPO': 'T', 'BMT': 'T', 'DTH': 'T', 'CTH': 'T',
    'TRP': 'W', 'TPL': 'W', 'TRO': 'W', 'DTR': 'W', 'HTR': 'W', 'LTR': 'W',
    'TYR': 'Y', 'TYQ': 'Y', 'TYS': 'Y', 'TYY': 'Y', 'TYB': 'Y', 'STY': 'Y', 'PTR': 'Y',
    'PAQ': 'Y', 'DTY': 'Y', 'IYR': 'Y', 'GHP': 'Y', 'D3P': 'Y', 'D4P': 'Y', 'OMZ': 'Y',
    'OMY': 'Y',
    'VAL': 'V', 'NVA': 'V', 'DVA': 'V', 'DIV': 'V', 'MVA': 'V',
    'SEC': 'U',
    'PYL': 'O',
    'XLE': 'J',
    'ACE': 'X', '3FG': 'X', 'UNK': 'X'
}

aminoacids1to3 = dict([[v, k] for k, v in aminoacids3to1.items()])
aminoacids1to3['A'] = 'ALA'
aminoacids1to3['N'] = 'ASN'
aminoacids1to3['R'] = 'ARG'
aminoacids1to3['D'] = 'ASP'
aminoacids1to3['C'] = 'CYS'
aminoacids1to3['Q'] = 'GLN'
aminoacids1to3['E'] = 'GLU'
aminoacids1to3['G'] = 'GLY'
aminoacids1to3['H'] = 'HIS'
aminoacids1to3['I'] = 'ILE'
aminoacids1to3['J'] = 'XLE'
aminoacids1to3['L'] = 'LEU'
aminoacids1to3['K'] = 'LYS'
aminoacids1to3['M'] = 'MET'
aminoacids1to3['F'] = 'PHE'
aminoacids1to3['O'] = 'PYL'
aminoacids1to3['P'] = 'PRO'
aminoacids1to3['S'] = 'SER'
aminoacids1to3['T'] = 'THR'
aminoacids1to3['U'] = 'SEC'
aminoacids1to3['W'] = 'TRP'
aminoacids1to3['Y'] = 'TYR'
aminoacids1to3['V'] = 'VAL'

'''
REGULAR AMINOACIDS IDENTIFICATION
'''
aminoacids_main3 = set(['ALA', 'ARG', 'ASN', 'ASP', 'CYS', 'GLN', 'GLU', 'GLX', 'GLY', 'HIS', 'ILE', 'LEU',
                        'LYS', 'MET', 'PHE', 'PRO', 'SER', 'THR', 'TRP', 'TYR', 'VAL', 'SEC', 'PYL', 'XLE'])

aminoacids_main1 = set(['A', 'R', 'N', 'D', 'C', 'Q', 'E', 'G', 'H', 'I', 'L', 'K', 'M', 'F', 'P', 'S', 'T', 'W', 'Y', 'V'])

'''
PROPERTIES
'''
aminoacids_surface = {
    'A': 115, 'C': 149, 'D': 170, 'E': 207, 'F': 230, 'G': 86,  'H': 206,
    'I': 187, 'K': 222, 'L': 192, 'M': 210, 'N': 184, 'P': 140, 'Q': 208,
    'R': 263, 'S': 140, 'T': 164, 'V': 161, 'W': 269, 'Y': 257,
}

aminoacids_polarity_boolean = {
    'A': False, 'C': False, 'D': True,  'E': True,  'F': False, 'G': False, 'H': True,
    'I': False, 'K': True,  'L': False, 'M': False, 'N': True,  'P': False, 'Q': True,
    'R': True,  'S': True,  'T': True,  'V': False, 'W': False, 'Y': True
}

'''
#
# DNA/RNA
#
'''
nucleic_main = set(['DA', 'A', 'DC', 'C', 'DG', 'G', 'DI', 'I', 'DT', 'T', 'DU', 'U'])
nucleic2to1  = {
    'DA': 'A', 'A': 'A', 'A44': 'A', '6HA': 'A', 'APN': 'A',
    'DC': 'C', 'C': 'C', '5CM': 'C', 'MCY': 'C', 'OMC': 'C', 'C43': 'C', '6HC': 'C',
    'CPN': 'C',
    'DG': 'G', 'G': 'G', 'OMG': 'G', 'G48': 'G', '6HG': 'G', 'GPN': 'G',
    'DI': 'I', 'I': 'I',
    'DT': 'T', 'T': 'T', '6HT': 'T', 'TPN': 'T',
    'DU': 'U', 'U': 'U', 'U36': 'U', '5IU': 'U',
    'N': 'N'
}

'''
CRYSTALOGRAPHIC METHODS
'''
crystal_method_has_resolution = set(['X-RAY DIFFRACTION', 'ELECTRON MICROSCOPY', 'NEUTRON DIFFRACTION',
                                     'FIBER DIFFRACTION', 'ELECTRON CRYSTALLOGRAPHY'])

crystal_method_not_resolution = set(['SOLUTION NMR', 'POWDER DIFFRACTION', 'SOLUTION SCATTERING', 'SOLID-STATE NMR',
                                     'INFRARED SPECTROSCOPY', 'FLUORESCENCE TRANSFER'])

crystal_method                = crystal_method_has_resolution.union(crystal_method_not_resolution)


'''
Elements
'''


class Element(object):
    def __init__(self, number, symbol, name):
        self.number = number
        self.symbol = symbol
        self.name   = name

element_dic = {
    'H':  Element(  1, 'H',  'Hydrogen'),    'He': Element(  2, 'He', 'Helium'),
    'Li': Element(  3, 'Li', 'Lithium'),     'Be': Element(  4, 'Be', 'Beryllium'),
    'B':  Element(  5, 'B',  'Boron'),       'C':  Element(  6, 'C',  'Carbon'),
    'N':  Element(  7, 'N',  'Nitrogen'),    'O':  Element(  8, 'O',  'Oxygen'),
    'F':  Element(  9, 'F',  'Fluorine'),    'Ne': Element( 10, 'Ne', 'Neon'),
    'Na': Element( 11, 'Na', 'Sodium'),      'Mg': Element( 12, 'Mg', 'Magnesium'),
    'Al': Element( 13, 'Al', 'Aluminium'),   'Si': Element( 14, 'Si', 'Silicon'),
    'P':  Element( 15, 'P',  'Phosphorus'),  'S':  Element( 16, 'S',  'Sulfur'),
    'Cl': Element( 17, 'Cl', 'Chlorine'),    'Ar': Element( 18, 'Ar', 'Argon'),
    'K':  Element( 19, 'K',  'Potassium'),   'Ca': Element( 20, 'Ca', 'Calcium'),
    'Sc': Element( 21, 'Sc', 'Scandium'),    'Ti': Element( 22, 'Ti', 'Titanium'),
    'V':  Element( 23, 'V',  'Vanadium'),    'Cr': Element( 24, 'Cr', 'Chromium'),
    'Mn': Element( 25, 'Mn', 'Manganese'),   'Fe': Element( 26, 'Fe', 'Iron'),
    'Co': Element( 27, 'Co', 'Cobalt'),      'Ni': Element( 28, 'Ni', 'Nickel'),
    'Cu': Element( 29, 'Cu', 'Copper'),      'Zn': Element( 30, 'Zn', 'Zinc'),
    'Ga': Element( 31, 'Ga', 'Gallium'),     'Ge': Element( 32, 'Ge', 'Germanium'),
    'As': Element( 33, 'As', 'Arsenic'),     'Se': Element( 34, 'Se', 'Selenium'),
    'Br': Element( 35, 'Br', 'Bromine'),     'Kr': Element( 36, 'Kr', 'Krypton'),
    'Rb': Element( 37, 'Rb', 'Rubidium'),    'Sr': Element( 38, 'Sr', 'Strontium'),
    'Y':  Element( 39, 'Y',  'Yttrium'),     'Zr': Element( 40, 'Zr', 'Zirconium'),
    'Nb': Element( 41, 'Nb', 'Niobium'),     'Mo': Element( 42, 'Mo', 'Molybdenum'),
    'Tc': Element( 43, 'Tc', 'Technetium'),  'Ru': Element( 44, 'Ru', 'Ruthenium'),
    'Rh': Element( 45, 'Rh', 'Rhodium'),     'Pd': Element( 46, 'Pd', 'Palladium'),
    'Ag': Element( 47, 'Ag', 'Silver'),      'Cd': Element( 48, 'Cd', 'Cadmium'),
    'In': Element( 49, 'In', 'Indium'),      'Sn': Element( 50, 'Sn', 'Tin'),
    'Sb': Element( 51, 'Sb', 'Antimony'),    'Te': Element( 52, 'Te', 'Tellurium'),
    'I':  Element( 53, 'I',  'Iodine'),      'Xe': Element( 54, 'Xe', 'Xenon'),
    'Cs': Element( 55, 'Cs', 'Caesium'),     'Ba': Element( 56, 'Ba', 'Barium'),
    'La': Element( 57, 'La', 'Lanthanum'),   'Ce': Element( 58, 'Ce', 'Cerium'),
    'Pr': Element( 59, 'Pr', 'Praseodymium'), 'Nd': Element( 60, 'Nd', 'Neodymium'),
    'Pm': Element( 61, 'Pm', 'Promethium'),  'Sm': Element( 62, 'Sm', 'Samarium'),
    'Eu': Element( 63, 'Eu', 'Europium'),    'Gd': Element( 64, 'Gd', 'Gadolinium'),
    'Tb': Element( 65, 'Tb', 'Terbium'),     'Dy': Element( 66, 'Dy', 'Dysprosium'),
    'Ho': Element( 67, 'Ho', 'Holmium'),     'Er': Element( 68, 'Er', 'Erbium'),
    'Tm': Element( 69, 'Tm', 'Thulium'),     'Yb': Element( 70, 'Yb', 'Ytterbium'),
    'Lu': Element( 71, 'Lu', 'Lutetium'),    'Hf': Element( 72, 'Hf', 'Hafnium'),
    'Ta': Element( 73, 'Ta', 'Tantalum'),    'W':  Element( 74, 'W', 'Tungsten'),
    'Re': Element( 75, 'Re', 'Rhenium'),     'Os': Element( 76, 'Os', 'Osmium'),
    'Ir': Element( 77, 'Ir', 'Iridium'),     'Pt': Element( 78, 'Pt', 'Platinum'),
    'Au': Element( 79, 'Au', 'Gold'),        'Hg': Element( 80, 'Hg', 'Mercury'),
    'Tl': Element( 81, 'Tl', 'Thallium'),    'Pb': Element( 82, 'Pb', 'Lead'),
    'Bi': Element( 83, 'Bi', 'Bismuth'),     'Po': Element( 84, 'Po', 'Polonium'),
    'At': Element( 85, 'At', 'Astatine'),    'Rn': Element( 86, 'Rn', 'Radon'),
    'Fr': Element( 87, 'Fr', 'Francium'),    'Ra': Element( 88, 'Ra', 'Radium'),
    'Ac': Element( 89, 'Ac', 'Actinium'),    'Th': Element( 90, 'Th', 'Thorium'),
    'Pa': Element( 91, 'Pa', 'Protactinium'), 'U':  Element( 92, 'U', 'Uranium'),
    'Np': Element( 93, 'Np', 'Neptunium'),   'Pu': Element( 94, 'Pu', 'Plutonium'),
    'Am': Element( 95, 'Am', 'Americium'),   'Cm': Element( 96, 'Cm', 'Curium'),
    'Bk': Element( 97, 'Bk', 'Berkelium'),   'Cf': Element( 98, 'Cf', 'Californium'),
    'Es': Element( 99, 'Es', 'Einsteinium'), 'Fm': Element(100, 'Fm', 'Fermium'),
    'Md': Element(101, 'Md', 'Mendelevium'), 'No': Element(102, 'No', 'Nobelium'),
    'Lr': Element(103, 'Lr', 'Lawrencium'),  'Rf': Element(104, 'Rf', 'Rutherfordium'),
    'Db': Element(105, 'Db', 'Dubnium'),     'Sg': Element(106, 'Sg', 'Seaborgium'),
    'Bh': Element(107, 'Bh', 'Bohrium'),     'Hs': Element(108, 'Hs', 'Hassium'),
    'Mt': Element(109, 'Mt', 'Meitnerium'),  'Ds': Element(110, 'Ds', 'Darmstadtium'),
    'Rg': Element(111, 'Rg', 'Roentgenium'), 'Cn': Element(112, 'Cn', 'Copernicium')
}


import numpy as np


def vectors_angle(a, b, degrees = False):
    def _vdot(a, b):
        if len(a.shape) == 1:
            return np.sum(a*b)
        elif len(a.shape) == 2:
            return np.sum(a*b, 1)

    def _vabs(a):
        if len(a.shape) == 1:
            return np.sqrt(np.sum(a**2))
        elif len(a.shape) == 2:
            return np.sqrt(np.sum(a**2, 1))

    a = np.asarray(a)
    b = np.asarray(b)

    if a.shape != b.shape:
        raise 'vectors have different shape'

    if not degrees:
        return np.arccos(_vdot(a, b) / (_vabs(a)*_vabs(b)))
    else:
        return np.degrees(np.arccos(_vdot(a, b) / (_vabs(a)*_vabs(b))))



import re, copy
import BioLib.Structure.PDB

class FTDock():
    '''
    '''

    static = None
    mobile = None
    gridSize = None
    searchAngleStep = 12
    surfaceThickness = 1.30
    internalDeterrentDalue = -15.00
    electrostatics = 1
    keepPerRotation = 3
    rotations = None
    totalSpan = None
    gridCellSpan = None
 
    decoys = []

    def __init__(self, ftdockFile):
        '''
        Call the parsing methods (parses an ftdock_global.dat file)
        '''
        self.__parse_info(ftdockFile)
        self.__parse_decoys(ftdockFile)

    def __parse_info(self, ftdockFile):
        '''
        '''
        ftdockfo = open(ftdockFile, 'r')
        for line in ftdockfo.readlines():
            static =  re.search('Static molecule\s+::\s+([\w\.]+)', line) 
            mobile = re.search('Mobile molecule\s+::\s+([\w\.]+)', line)
            gridSize = re.search('Global grid size\s+::\s+([\d\.-]+)', line)
            searchAngleStep = re.search('Global search angle step\s+::\s+([\d\.-]+)', line)
            surfaceThickness = re.search('Global surface thickness\s+::\s+([\d\.-]+)', line)
            internalDeterrentDalue = re.search('Global internal deterrent value\s+::\s+([\d\.-]+)', line)
            electrostatics = re.search('Electrostatics\s+::\s+(\w+)', line)
            keepPerRotation = re.search('Global keep per rotation\s+::\s+([\d\.-]+)', line)
            rotations = re.search('Global rotations\s+::\s+([\d\.-]+)', line)
            totalSpan = re.search('Global total span \(angstroms\)\s+::\s+([\d\.-]+)', line)
            gridCellSpan = re.search('Global grid cell span \(angstroms\)\s+::\s+([\d\.-]+)', line)

            if static: self.static = static.group(1)
            elif mobile: self.mobile = mobile.group(1)
            elif gridSize: self.gridSize = gridSize.group(1)
            elif searchAngleStep: self.searchAngleStep = searchAngleStep.group(1)
            elif surfaceThickness: self.surfaceThickness = surfaceThickness.group(1)
            elif internalDeterrentDalue: self.internalDeterrentDalue = internalDeterrentDalue.group(1)
            elif electrostatics:
                if electrostatics.group(1) != 'on': self.electrostatics = 0
            elif keepPerRotation: self.keepPerRotation = keepPerRotation.group(1)
            elif rotations: self.rotations = rotations.group(1)
            elif totalSpan: self.totalSpan = totalSpan.group(1)
            elif gridCellSpan: self.gridCellSpan = gridCellSpan.group(1)

    def __parse_decoys(self, ftdockFile):
        '''
        '''
        ftdockfo = open(ftdockFile, 'r')
        for line in ftdockfo.readlines():
            if line[0:6] == 'G_DATA':
                self.decoys.append(FTDecoy(int(line[8:14]), float(line[27:34]), float(line[40:49]), int(line[55:59]), int(line[59:64]), int(line[64:69]), int(line[75:79]), int(line[79:83]), int(line[83:])))
   
    def get_grid_size(self):
        return self.gridSize

    def get_decoys(self):
        return self.decoys
 
    def get_decoy_by_id(self,id):

        for decoy in self.decoys:
            if decoy.get_id() == id:
                return decoy
        return None

    def get_grid_total_span(self, static, mobile):
        '''
        Calculate the total span of the structures
        '''
        return 1 + ((static.get_radius() + mobile.get_radius()) * 2)

    def get_one_span(self, static, mobile):
        '''
        Calculate the GRID one span
        '''
        return self.get_grid_total_span(static, mobile)/float(self.gridSize) 

    def get_static_structure(self, static):
        '''
        Gets the static structure of the decoys (same in all decoys)
        '''
        staticDecoy = copy.deepcopy(static)
        staticDecoy.translate_onto_origin()
        return staticDecoy

    def get_mobile_structure(self, static, mobile, decoy):
        '''
        Gets the structure of each decoy (ftdock only relocates the mobile structure)
        '''
        mobileDecoy = copy.deepcopy(mobile)
        mobileDecoy.translate_onto_origin()

        translationVector = decoy.get_corrected_tranlation_vector(self.get_one_span(self.get_static_structure(static), mobileDecoy))
        twistVector = decoy.get_rotation_vector()
        mobileDecoy.relocate(translationVector=translationVector, twistVector=twistVector)        

        return mobileDecoy

    def get_ligand_RMSD(self, static, mobile, native, decoy):
        '''
        Gets the ligand RMSD between one decoy and the original (native) mobile structure
        '''
        originalMobile = copy.deepcopy(native)
        staticDecoy = copy.deepcopy(static)
        mobileDecoy = self.get_mobile_structure(static, mobile, decoy)

        centerTraslationVector = staticDecoy.translate_onto_origin()
        originalMobile.relocate(translationVector=centerTraslationVector)

        return originalMobile.get_RMSD(mobileDecoy)

    def get_decoys_ligand_RMSD(self, static, mobile, native):
        '''
        Gets a dictionary of the ligand RMSD between each decoy and the original (native) mobile structure
        '''
        ligandRMSD = {}
        for decoy in self.get_decoys():
            ligandRMSD[decoy.get_id()] = self.get_ligand_RMSD(static, mobile, native, decoy)
            print str(decoy.get_id())+'\t'+str(ligandRMSD[decoy.get_id()])
        return ligandRMSD
    
    def print_RMSD(self, static, mobile, native, RMSDFile):
        '''
        Prints the ligand RMSD of each decoy in a tab file
        '''
        RMSDfo = open(RMSDFile, 'w')
        ligandRMSD = self.get_decoys_ligand_RMSD(static, mobile, native)
        for key, value in sorted(ligandRMSD.iterkeys(), key=lambda (k,v): (v,k)):
            RMSDfo.write('%d\t%.3f\n' % (key, value))
        RMSDfo.close()

    def print_decoys(self, static, mobile, pdbFile, singlePDB=False):
        '''
        Print each decoy in a PDB format
        @singlePDB = Prints each decoy as a model in a single PDB file
        '''
        staticDecoy = self.get_static_structure(static)
        decoyList = []
        for decoy in self.get_decoys():
            mobileDecoy = self.get_mobile_structure(static, mobile, decoy)
            if singlePDB:
                DecoyList.append((staticDecoy, mobileDecoy))
            else:
                PDB.write_pdb((staticDecoy, mobileDecoy), pdbFile+'_'+str(decoy.get_id())+'.pdb' , multiChain=True, multiModel=False)
        if singlePDB:
            PDB.write_pdb(decoyList, pdbFile+'.pdb', multiChain=True, multiModel=True)
    
    def __str__(self):
        '''
        '''
        return "Static molecule: %s\nMobile molecule: %s\nGlobal grid size: %s\nGlobal search angle step: %s\nGlobal surface thickness: %s\nGlobal internal deterrent value: %s\nElectrostatics: %s\nGlobal keep per rotation: %s\nGlobal rotations: %s\nGlobal total span (angstroms): %s\nGlobal grid cell span (angstroms): %s" % (self.static, self.mobile, self.gridSize, self.searchAngleStep, self.surfaceThickness, self.internalDeterrentDalue, self.electrostatics, self.keepPerRotation, self.rotations, self.totalSpan, self.gridCellSpan)


class FTDecoy():
    '''
    '''
    def __init__(self, id, SCscore, ESratio, x, y, z, ztwist, theta, phi):
        '''
        '''
        self.id = id
        self.SCscore = SCscore
        self.ESratio = ESratio
        self.x = x
        self.y = y
        self.z = z
        self.ztwist = ztwist
        self.theta = theta
        self.phi = phi

    def get_id(self):
        return self.id

    def get_SCscore(self):
        return self.SCscore

    def get_ESratio(self):
        return self.ESratio

    def get_tranlation_vector(self):
        return (self.x, self.y, self.z)
   
    def get_corrected_tranlation_vector(self, one_span):
        '''
        Gets the corrected translation vector (needed for ftdock)
        '''
        return (self.x * one_span, self.y * one_span, self.z * one_span)

    def get_rotation_vector(self):
        return (self.ztwist, self.theta, self.phi)

    def __str__(self):
        '''
        '''
        return ('G_DATA\t%d\t%d\t%.3f\t%s%s%s\t%s%s%s') % (self.id, self.SCscore, self.ESratio, str(self.x).rjust(5), str(self.y).rjust(5), str(self.z).rjust(5), str(self.ztwist).rjust(5), str(self.theta).rjust(5), str(self.phi).rjust(5))



'''
@file: Sequence.py

@author: Jaume Bonet
@mail:   jaume.bonet@gmail.com
@date:   05/2013

@ [oliva's lab](http://sbi.imim.es)

@class: Sequence
'''
import re
import copy
import hashlib
import difflib
from collections import Counter


class Sequence(object):
    '''
    A simple container for sequences.
    Includes the sequence identifier, the sequence information and the sequence
    itself.
    Default comparison performed between sequences by length.
    Iterable.

    '''
    AVAILABLE_FORMATS          = set(['TAB', 'FASTA', 'PIR'])
    TOKEN_FORMATS              = set(['BINARY'])
    GAP_DEFINITION             = '[-x]'
    NEEDLEMAN_WUNSCH           = set(['NEEDLEMAN-WUNSCH', 'NW'])
    AVAILABLE_ALIGN_ALGORITHMS = set()

    AVAILABLE_ALIGN_ALGORITHMS.update(NEEDLEMAN_WUNSCH)

    def __init__(self, sequence_id = '', sequence = ''):
        '''
        @param:    sequence_id
        @pdef:     Name of the sequence.
        @pdefault: ''
        @ptype:    {String}

        @param:    sequence
        @pdef:     Sequence.
        @pdefault: ''
        @ptype:    {String} or {List}

        @raises: {AttributeError} if sequence is not {String} or {List}.
        '''
        self._seqID = ''
        self._info  = ''
        self.id     = sequence_id

        self._sequence = ''
        self._gapped   = False
        self.sequence  = sequence

    ##############
    # ATTRIBUTES #
    ##############
    @property
    def id(self):
        '''
        Sequence identifier.

        @return: {String}
        '''
        return self._seqID

    @id.setter
    def id(self, value):
        '''
        @param:    id
        @pdef:     Name of the sequence.
        @pdefault: ''
        @ptype:    {String}
        '''
        if bool(re.search(r'\s', value)):
            self._seqID = value.split()[0]
            self._info  = ' '.join(value.split()[1:])
        else:
            self._seqID = value
            self._info  = ''

    @property
    def sequence(self):
        '''
        Sequence.

        @return: {String}
        '''
        return self._sequence

    @sequence.setter
    def sequence(self, value):
        '''
        @param:    sequence
        @pdef:     Sequence.
        @pdefault: ''
        @ptype:    {String} or {List}
        '''
        if isinstance(value, str):
            self._sequence = value
        elif isinstance(value, list):
            self._sequence = ''.join(value)
        else:
            raise AttributeError('sequence attribute must be string or list.')

        self._gapped = bool(re.search(self.GAP_DEFINITION, self.sequence))

    @property
    def info(self):
        '''
        Sequence description.

        @return: {String}
        '''
        return self._info

    @info.setter
    def info(self, value):
        '''
        @param:    info
        @pdef:     Description of the sequence.
        @pdefault: ''
        @ptype:    {String}
        '''
        self._info = value

    @property
    def full_identifier(self):
        '''
        Full name of the sequence.

        @return: {String}
        '''
        return ' '.join([self.id, self.info]).strip()

    @property
    def md5(self):
        '''
        Creates an md5 of the sequence as a unique identifier.

        @return: {String}
        '''
        pre = self.sequence[:3]
        md5 = hashlib.sha224(self.sequence).hexdigest()
        pos = self.sequence[-3:]
        return pre + md5 + pos

    ############
    # BOOLEANS #
    ############
    @property
    def is_gapped(self):
        '''
        Determines it the sequence contains gaps.

        @return: {Boolean}
        '''
        return self._gapped

    ###########
    # METHODS #
    ###########
    def chunk(self, size = 60):
        '''
        Split the sequence in segments of up to 'size' elements.

        @param:    size
        @pdef:     Max size of the chunks.
        @ptype:    {Integer}
        '''
        return [self.sequence[i:i+size] for i in range(0, len(self), size)]

    def contains(self, sequence):
        '''
        Checks if a given sequence is contained inside.

        @param:    sequence
        @pdef:     Sequence to compare with.
        @ptype:    {String}, {List} or {Sequence}

        @raises: {AttributeError} if sequence is the wrong type.
        @return: {Boolean}
        '''
        if isinstance(sequence, basestring):
            return bool(re.search(sequence, self.sequence))
        elif isinstance(sequence, list):
            return bool(re.search(''.join(sequence), self.sequence))
        elif isinstance(sequence, Sequence):
            return bool(re.search(sequence.sequence, self.sequence))
        else:
            return AttributeError('sequence must be string, list or Sequence')

    def contained(self, sequence):
        '''
        Checks if the sequence is contained inside a given one.

        @param:    sequence
        @pdef:     Sequence to compare with.
        @ptype:    {String}, {List} or {Sequence}

        @raises: {AttributeError} if sequence is the wrong type.
        @return: {Boolean}
        '''
        if isinstance(sequence, basestring):
            return bool(re.search(self.sequence, sequence))
        elif isinstance(sequence, list):
            return bool(re.search(self.sequence, ''.join(sequence)))
        elif isinstance(sequence, Sequence):
            return bool(re.search(self.sequence, sequence.sequence))
        else:
            return AttributeError('sequence must be string, list or Sequence')

    def get_matching_blocks(self, sequence):
        '''
        Returns each identical block between the sequence and a given one

        @param:    sequence
        @pdef:     Sequence to compare with.
        @ptype:    {String}, {List} or {Sequence}

        @raises: {AttributeError} if sequence is the wrong type.
        @return: {List} of {String}
        '''
        if isinstance(sequence, basestring):
            pass
        elif isinstance(sequence, list):
            sequence = ''.join(sequence)
        elif isinstance(sequence, Sequence):
            sequence = sequence.sequence
        else:
            return AttributeError('sequence must be string, list or Sequence')

        m = difflib.SequenceMatcher(None, self.sequence, sequence)
        b = []
        for match in m.get_matching_blocks():
            i, j, n = match
            b.append(self[i:i+n])
        return b

    def align(self, sequence, algorithm = 'NW', similarity_matrix = 'BLOSUM62',
              gap_init = None, gap_penalty = None):
        '''
        Aligns the sequence with a given one.

        @param:    sequence
        @pdef:     Sequence to compare with.
        @ptype:    {String}, {List} or {Sequence}

        @@param:   algorithm
        @pdef:     Algorithm to use for the alignment
        @pdefault: 'needleman_wunsch'
        @ptype:    {String}

        @param:    similarity_matrix
        @pdef:     Scoring matrix for the alignment.
        @pdefault: 'BLOSUM62'
        @ptype:    {String}

        @param:    gap_init
        @pdef:     Value of initiating a gap
        @pdefault: Gap value of the similarity_matrix
        @ptype:    {Integer}

        @param:    gap_penalty
        @pdef:     Value of extending a gap
        @pdefault: Gap value of the similarity_matrix
        @ptype:    {Integer}

        @raises: {AttributeError} if sequence is the wrong type or if a non-
                 available alignment method is requested.
        @return: {SeqAli}
        '''
        if algorithm.upper() not in Sequence.AVAILABLE_ALIGN_ALGORITHMS:
            raise AttributeError('Alignment algorithm not available')
        if isinstance(sequence, basestring):
            pass
        elif isinstance(sequence, list):
            sequence = ''.join(sequence)
        elif isinstance(sequence, Sequence):
            sequence = sequence.sequence
        else:
            return AttributeError('sequence must be string, list or Sequence')

        if algorithm.upper() in Sequence.NEEDLEMAN_WUNSCH:
            from alignment.Needleman_Wunsch import needleman_wunsch as nw
            return nw(self.sequence, sequence, similarity_matrix,
                      gap_init, gap_penalty)

    def format(self, format = 'TAB'):
        '''
        Formats the sequence.

        @param:    format
        @pdef:     Output format for the sequence
        @pdefault: 'TAB'
        @ptype:    {String}

        @raises: {AttributeError} for unknown formats
        @return: {String}
        '''
        if format.upper() not in Sequence.AVAILABLE_FORMATS:
            raise AttributeError('format option not available')

        if format.upper() == 'TAB':
            return '{0.id}\t{0.sequence}'.format(self)
        elif format.upper() == 'FASTA':
            return '>{0.id}\n{0.sequence}'.format(self)
        elif format.upper() == 'PIR':
            data = ['>P1;{0.id}'.format(self),
                    'sequence:{0.id}:::::::0.00: 0.00'.format(self)]
            data.extend(self.chunk())
            return '\n'.join(data) + '*'

    def tokenize(self, token_coding = 'BINARY'):
        '''
        Re-codifies the sequence according to a given codification.

        @param:    token_coding
        @pdef:     codification to tokenize.
        @pdefault: 'BINARY'
        @ptype:    {String}
        '''
        if token_coding.upper() not in Sequence.TOKEN_FORMATS:
            raise AttributeError('Unrecognized token coding.')
        if token_coding.upper() == 'BINARY':
            tc = ['\D', '1', '-', '0']
            return re.sub(tc[0], tc[1], re.sub(tc[2], tc[3], self._sequence))

    def items_frequency(self):
        '''
        Displays the frequency of the different items in the sequence.

        @return: {Dictionary}
        '''
        c = Counter(re.sub(Sequence.GAP_DEFINITION, '', self._sequence))
        l = float(len(self))
        return dict([(x, y/l) for x, y in c.iteritems()])

    def duplicate(self, new_id = None):
        '''
        Creates a new copy of the sequence.

        @param:    new_id
        @pdef:     a new identifier can be set in the copy
        @ptype:    {String}

        @return: {Sequence}
        '''
        s = copy.deepcopy(self)
        if new_id is not None:
            s.id = new_id
        return s

    def do_ungap(self):
        '''
        Removes the gaps in the sequence.
        '''
        if self._gapped:
            self.sequence = re.sub(self.GAP_DEFINITION, '', self.sequence)

    def ungapped(self):
        '''
        Returns a copy of the sequence without gaps.
        Does not alter the sequence.

        @return: {String}
        '''
        return re.sub(self.GAP_DEFINITION, '', self.sequence)

    def append(self, sequence):
        '''
        Extends the sequence.

        @param:    sequence
        @pdef:     Sequence to compare with.
        @ptype:    {String}, {List} or {Sequence}

        @raises: {AttributeError} if sequence is the wrong type.
        '''
        if isinstance(sequence, basestring):
            pass
        elif isinstance(sequence, list):
            sequence = ''.join(sequence)
        elif isinstance(sequence, Sequence):
            sequence = sequence.sequence
        else:
            return AttributeError('sequence must be string, list or Sequence')

        self.sequence = self.sequence + sequence

    #################
    # CLASS METHODS #
    #################
    def __len__(self):
        return len(self._sequence)

    def __eq__(self, other):
        if isinstance(other, Sequence):
            return self.sequence == other.sequence
        return NotImplemented

    def __ne__(self, other):
        result = self.__eq__(other)
        if result is NotImplemented:
            return result
        return not result

    def __lt__(self, other):
        if isinstance(other, Sequence):
            return len(self) < len(other)
        return NotImplemented

    def __gt__(self, other):
        if isinstance(other, Sequence):
            return len(self) > len(other)
        return NotImplemented

    def __getitem__(self, key):
        try:
            int(key)
            return self._sequence[int(key)]
        except:
            if not isinstance(key, slice):
                raise TypeError
            else:
                return self._sequence[key]

    def __iter__(self):
        for s in self._sequence:
            yield s

    def __repr__(self):
        return '<{0.__class__.__name__}: [{0.id}, {0.sequence}]>'.format(self)

    def __str__(self):
        return '{0.id}\t{0.sequence}'.format(self)



"""
Residue

author: jbonet
date:   02/2013

@oliva's lab
"""

import numpy         as np
import scipy.spatial as sp

from SBI import SBIglobals

class Residue(object):
    """
    A {Residue} collects a series of {Atom}
    """

    def __init__(self, number, version, Rtype, mode):
        """
        @type  number: Integer
        @param number: Residue number

        @type  version: Char
        @param version: Optional char used on pdbs to change count

        @type  type: String
        @param type: Residue type

        @type  mode: String
        @param mode: Residue mode: ATOM or HETATM
        """
        self._number    = int(number)
        self._version   = version
        self._type      = Rtype
        self._mode      = mode

        # self._atoms               = []
        self._backbone_coordinates  = None
        self._backbone_atoms        = []
        self._sidechain_coordinates = None
        self._sidechain_atoms       = []

    #
    # ATTRIBUTES
    #
    @property
    def number(self):
        """
        Residue number
        @rtype: Integer
        """
        return int(self._number)

    @number.setter
    def number(self, value):
        """
        @type value: Integer
        """
        self._number = int(value)

    @property
    def version(self):
        """
        Residue version
        @rtype: CHAR
        """
        return self._version

    @version.setter
    def version(self, value):
        self._version = value

    @property
    def identifier(self):
        """
        Combo number + version
        @rtype: String
        """
        return str(self.number) + self.version

    @property
    def type(self):
        """
        Residue type
        @rtype: String
        """
        return self._type

    @property
    def mode(self):
        """
        Residue mode: ATOM or HETATM
        @rtype: String
        """
        return self._mode

    @property
    def atoms(self):
        """
        List of {Atom} objects of the {Residue}
        @rtype: List of {Atom}
        """
        # return self._atoms
        return self._backbone_atoms + self._sidechain_atoms

    @property
    def backbone_atoms(self): return self._backbone_atoms
    @property
    def sidechain_atoms(self): return self._sidechain_atoms
    @property
    def all_coordinates(self):
        """
        Coordinates of all the {Atom}s in the {Residue}
        @rtype: numpy.matrix
        """
        if self._backbone_coordinates is not None and self._sidechain_coordinates is not None:
            return np.vstack((self._backbone_coordinates, self._sidechain_coordinates))
        else:
            if self._backbone_coordinates is not None:
                return self._backbone_coordinates
            elif self._sidechain_coordinates is not None:
                return self._sidechain_coordinates 

    @property
    def first_atom_number(self):
        """
        Returns the number of the first {Atom} of the {Residue}
        @rtype: Integer
        """
        # return self._atoms[0].number
        return self.atoms[0].number

    @property
    def last_atom_number(self):
        """
        Returns the number of the last {Atom} of the {Residue}
        @rtype: Integer
        """
        # return self._atoms[-1].number
        return self.atoms[-1].number

    @property
    def geometric_center(self):
        """
        Returns the geometric central position of the {Residue}
        @rtype numpy.array
        """
        return sum(self.all_coordinates)/self.all_coordinates[:,1].size

    #
    # ADVANCED GETTERS & SETTERS
    #
    def add_atom(self, atom):
        """
        Adds a new {Atom} to the {Residue}
        This includes filling the all_coordinates attribute

        @type  atom: {Atom}
        @param atom: New {Atom} added to the {Residue}
        """
        # self._atoms.append(atom)
        if atom.is_backbone: self._backbone_atoms.append(atom)
        else:                self._sidechain_atoms.append(atom)
        self._add_to_matrix(atom)

    #
    # METHODS
    #
    def normalize(self):
        raise NotImplementedError

    def renumerate_atoms(self, first_atom_number):
        """
        Given the number for the first atom, renumerates the rest accordingly
        @type  value: Integer
        """
        for atom in self.atoms:
            atom.number = first_atom_number
            first_atom_number +=  1

    def rotate(self, matrix = None):
        """
        Rotates a {Residue} according to a given matrix

        @type matrix: numpy.matrix
        """
        if matrix is None: matrix = np.identity(3)
        self._backbone_coordinates  = None
        self._sidechain_coordinates = None
        for atom in self.atoms:
            atom.rotate(matrix = matrix)
            self._add_to_matrix(atom)

    def translate(self, vector):
        """
        Translates a {Residue} according to a translational vector

        @type vector: numpy.array
        """
        if vector is None: vector = np.zeros(3, float)
        self._backbone_coordinates  = None
        self._sidechain_coordinates = None
        for atom in self.atoms:
            atom.translate(vector = vector)
            self._add_to_matrix(atom)

    def reposition(self, matrix = None, vector = None):
        """
        Rotates and translates the {Residue} according to a matrix and translational vector

        @type matrix: numpy.matrix

        @type vector: numpy.array
        """
        if matrix is None: matrix = np.identity(3, float)
        if vector is None: vector = np.zeros(3, float)
        SBIglobals.alert('deepdebug', self, 'Reposition residue {0.type}:{0.number}'.format(self))

        self._backbone_coordinates  = None
        self._sidechain_coordinates = None
        for atom in self.atoms:
            SBIglobals.alert('deepdebug', self, 'Atom {0.name} {0.is_backbone}'.format(atom))
            atom.rotate(matrix = matrix)
            atom.translate(vector = vector)
            self._add_to_matrix(atom)

    def _add_to_matrix(self, atom):
        def add_to_matrix(matrix, atom):
            if matrix is None:
                matrix = np.array(np.zeros(3))
                matrix = np.vstack((matrix, atom.coordinates))
                matrix = np.delete(matrix, 0, 0)
            else:
                matrix = np.vstack((matrix, atom.coordinates))
            return matrix
        if atom.is_backbone:
            self._backbone_coordinates  = add_to_matrix(self._backbone_coordinates, atom)
        else:
            self._sidechain_coordinates = add_to_matrix(self._sidechain_coordinates, atom)

    def distance(self, residue, dist_type = 'min'):
        """
        Calculates the distance between two residues

        @type  residue: {Residue}
        @param residue: Residue to which we want to find the distance

        @type  dist_type: String
        @param dist_type: Type of distance to evaluate
        @limit dist_type: Accepted: distance.types (min, geometric)

        @rtype: (AT1, AT2, DISTANCE)
        """
        if dist_type not in set(['min','geometric', 'backbone']): dist_type = 'min'

        if dist_type.strip().lower() == 'min':
            return self._distance_min(residue = residue)
        if dist_type.strip().lower() == 'geometric':
            return self._distance_geometric_center(residue = residue)
        if dist_type.strip().lower() == 'backbone':
            return self._distance_min_backbone(residue = residue)

    def follows(self, residue):
        number0  = self.number
        version0 = self.version
        number1  = residue.number
        version1 = residue.version
        print(number0, number1 + 1)
        if int(number0) == int(number1 + 1): return True
        if int(number0) == int(number1):
            if version0 == ' ': version0 = '@'
            if version1 == ' ': version1 = '@'
            if ord(version0) == ord(version1) + 1: return True
        return False

    def is_followed(self, residue):
        number0  = self.number
        version0 = self.version
        number1  = residue.number
        version1 = residue.version
        if int(number0) == int(number1 - 1): return True
        if int(number0) == int(number1):
            if version0 == ' ': version0 = '@'
            if version1 == ' ': version1 = '@'
            if ord(version0) == ord(version1) - 1: return True
        return False

    def identifier_distance(self, residue):
        number0  = self.number
        version0 = self.version
        number1  = residue.number
        version1 = residue.version 

        if number0 != number1:
            return number1-number0
        else:
            return abs(ord(version1)-ord(version0))

    def _distance_min(self, residue):
        """
        Calculates the minimum distance between two residues

        @type  residue: {Residue}
        @param residue: Residue to which we want to find the distance

        @rtype: 
        """
        distances = sp.distance.cdist(self.all_coordinates, residue.all_coordinates)
        index     = np.unravel_index(distances.argmin(), distances.shape)
        return (self.atoms[index[0]], residue.atoms[index[1]], distances.min())

    def _distance_min_backbone(self, residue):
        if self._backbone_coordinates is None or residue._backbone_coordinates is None:
            return (None, None, -1)
        else:
            distances = sp.distance.cdist(self._backbone_coordinates, residue._backbone_coordinates)
            index     = np.unravel_index(distances.argmin(), distances.shape)
            return (self._backbone_atoms[index[0]], residue._backbone_atoms[index[1]], distances.min())

    def _distance_geometric_center(self, residue):
        """
        Calculates the distance between the geometric center of two residues

        @type  residue: {Residue}
        @param residue: Residue to which we want to find the distance

        @rtype: 
        """
        return (None, None, sp.distance.euclidean(self.geometric_center, residue.geometric_center))

    #
    # OVERRIDE DEFAULT OBJECT METHODS
    #
    def __len__(self):
        """
        number of atoms in the Residue
        @rtype: Integer
        """
        return len(self.atoms)

    def __hash__(self): 
        return '<{0.__class__.__name__}: [{0.type}, {0.number}, {0.version}]>'.format(self).__hash__()
    def __eq__(self, other):
        return (self.type, self.number, self.version) == (other.type, other.number, other.version)

    def __repr__(self):
        repre = []
        repre.append('<{0.__class__.__name__}: [{0.type}, {0.number}, {0.version}]>'.format(self))
        for atom in self.atoms:
            repre.append("\t" + repr(atom))
        return "\n".join(repre)



import sys, os, argparse, gzip

from SBI.beans           import Path
from SBI.databases import Enzymelink

from archdb.src import Source, Enzyme
from archdb.src import start_transaction, end_transaction


#
# deleted and transfered entries should have had their parents set by inheritance. they were not
# THIS MUST BE TAKEN INTO ACCOUNT WHEN REDOING THIS
#
#
# User Options
def options(*args, **kwds):
    parser = argparse.ArgumentParser(prog = 'Enzyme2SQL',  formatter_class = argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-d', '--database',     dest     = "database", 
                        action = "store",       required = True,  help = "Destination Directory for Enzyme database.")
    parser.add_argument('-s', '--sql',          dest     = "sqlfile", 
                        action = "store",       required = True,  help = "Output SQL file")
    parser.add_argument('-k', '--skipdownload', dest     = "skip_download", 
                        action = "store_true",  default  = False,  help = "Skips downloading the database.")
    parser.add_argument('-v', '--verbose',      dest     = "verbose", 
                        action = "store_true",  default  = False, help = "Verbose Mode.")
    options = parser.parse_args()
    return options

def Enzyme2SQL(database, sqlfile, skip_download, verbose):
    enzyme_connect = Enzymelink(local = database)
    newsource      = None
    if not skip_download:
        if verbose: sys.stderr.write("Downloading Enzyme database to {0} ...\n".format(database))
        enzyme_connect.download()
        newsource = Source(name = 'enzyme', source = enzyme_connect.source)
        if verbose: sys.stderr.write("Download Finished.\n")
    else:
        if verbose: sys.stderr.write("Using previously downloaded database.\n")

    if verbose: sys.stderr.write("Parsing Enzyme.\n")
    if verbose: sys.stderr.write("Writing {0} ....\n".format(sqlfile))
    Path.mkdir(os.path.split(os.path.abspath(sqlfile))[0])
    sql_fd = gzip.open(sqlfile, 'wb')
    sql_fd.write(start_transaction())
    if newsource is not None:
        sql_fd.write(newsource.toSQL())

    transfers = []
    for enz_line in enzyme_connect.localEnzymes:
        newenz = Enzyme(inline = enz_line)
        sql_fd.write(newenz.toSQL())
        if newenz.has_transfers:
            transfers.append(newenz.transfered2SQL())

    sql_fd.write("".join(transfers))
    sql_fd.write(end_transaction())
    sql_fd.close()
    if verbose: sys.stderr.write("End execution.\n")

if __name__ == "__main__":
    options = options()
    Enzyme2SQL(options.database, options.sqlfile, options.skip_download, options.verbose)


import sys
import argparse
import os
import re
from Bio import SeqIO as SeqIE
from Bio import ExPASy
from Bio.Seq import Seq
from Bio import SeqRecord
from Bio.Alphabet import IUPAC
import ConfigParser
import itertools
import shutil
import subprocess
import time
import random
import numpy as np
import scipy.stats as st
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt


# Add '.' to sys.path
src_path = os.path.abspath(os.path.dirname(__file__))
sys.path.append(src_path)
script_path = src_path+"/../scripts"
sys.path.append(script_path)

# Read configuration file
config = ConfigParser.ConfigParser()
config_file = os.path.join(script_path, 'config.ini')
config.read(config_file)

# Add SBI library path to sys.path
src_path = os.path.join(config.get('Paths','modppi_path'),config.get('Paths','sbi_library_path'))
sys.path.append(src_path)

# Add functions path to sys.path    
src_path = os.path.join(config.get('Paths','modppi_path'),config.get('Paths','functions_path'))
sys.path.append(src_path)

# Import functions
import functions
from SeqIO import *
from functions import *
from SBI.external.blast import blast_parser
from SBI.structure.contacts import Complex
from SBI.sequence import Sequence
from SBI.structure import PDB

def fileExist (file):
 return os.path.exists(file) and os.path.isfile(file)


def reduce_outliers_lstsq(y,x,m,c,t):
    yy=[]
    xx=[]
    d={}
    n = int(len(x)*t/100)
    out=0
    for i in xrange(len(x)):
        b = m*x[i] + c
        d.setdefault(i,(abs(y[i] - b)))
    for key, value in sorted(d.iteritems(), key=lambda (k,v): (v,k),reverse=True):
        out = out + 1
        if out > n:
           yy.append(y[key])
           xx.append(x[key])
        else:
           sys.stdout.write("Outlier: %10d \tY= %f \tX= %f \tRMSD= %f\n"%(key,y[key],x[key],value))
    return (yy,xx)



def parse_user_arguments(*args, **kwds):
    parser = argparse.ArgumentParser(
        description = 'Selection and energy comparison of  WT versus mutant forms of clusters of protein-protein interactions',
        epilog      = '@Oliva\'s lab 2018')
    parser.add_argument('-d', '--folder', dest = 'folder', action = 'store',
                        help = 'Input directory with MODPPI results')
    parser.add_argument('-g','--ddG',dest='ddg',action = 'store',default=None,
                        help = 'SKEMPI parsed file of affinities')
    parser.add_argument('-e', '--Energy_tested', dest = 'score', action = 'store',default="ddG_mean",
                        help = 'Energy to be used in the test')
    parser.add_argument('-o', '--output_directory', dest = 'outdir', action = 'store', default = 'selected_clusters',
                        help = 'Output directory (default is ModPPI_models)')
    parser.add_argument('-l','--label',dest='label',action = 'store',default='',
                        help = 'Label to identify the output "Compare_ddG_with_average_"')
#    parser.add_argument('-n', '--number_of_models', dest = 'nmodels', action = 'store', default = 1, type=int,
#                        help = 'Minimum number of models select the cluster for analysis (default is 1)')
    parser.add_argument('-i', '--interface_percentil_mutants', dest = 'ipercentil', action = 'store', default = 1.0, type=float,
                        help = 'Minimum percentage of models with mutation in the interface to select the cluster for analysis (default is 0.0)')
    parser.add_argument('-p', '--p_value_threshold', dest = 'p_value', action = 'store', default = 0.05, type=float,
                        help = 'P-value threshold on the comparison of WT-mutant to select the cluster for analysis (default is 0.05)')
    parser.add_argument('-r', '--cluster_rank', dest = 'rank', action = 'store', default = 1, type=int,
                        help = 'Cluster ranking by size (default is 1)')
    parser.add_argument('-lt','--outliers_threshold',dest='outlier_threshold',action = 'store',default=None,
                        help = 'Free Energy Perturbation Outliers energy (limits max and min of FEP)')
    parser.add_argument('-lp','--outliers_deviation',dest='outlier_percentil',action = 'store',default='0.0',
                        help = 'Percentage of Outliers (neglects a percentage of data, default is 0 and uses all)')
    parser.add_argument('-s','--show_canvas',dest='canvas',action = 'store_true',
                        help = 'Show canvas figure after execution (not shown by default)')
    parser.add_argument('-fi','--format_image',dest='format',action = 'store',default='png',
                        help = 'Format of the image in the output file: png(default), pdf, ps, eps and svg ')
    parser.add_argument('-k','--factor_lambda',dest='factor_lambda',action = 'store',default='1.0',type=float,
                        help = 'Factor factor_lambda to multiply the energies (default is 1) ')
    parser.add_argument('-c','--clean_null',dest='clean_null',action = 'store',default=None,
                        help = 'Clean ddG values near 0.0 because they are caused by errors (default is None, otherwise select a threshold near 0, i.e. 0.1)')
    parser.add_argument('-v', '--verbose', dest = 'show', action = 'store_true',
                        help = 'Flag for verbose mode (default is False)')
    options = parser.parse_args()
    return options



def main():

 options=parse_user_arguments()

 files       = os.listdir(options.folder)
 score       = options.score
# nmodels     = options.nmodels
 p_threshold = options.p_value
 percentil   = options.ipercentil
 outdir      = options.outdir
 input_dir   = options.folder
 rank        = options.rank
 verbose     = options.show
 factor      = options.factor_lambda
 label_out   = options.label

 if not os.path.exists(outdir): os.makedirs(outdir)

 if verbose: sys.stdout.write("Parsing %s\n"%(options.ddg))
 ddg_dict={}
 if not fileExist(options.ddg):
    sys.stderr.write("File %s not found\n"%options.ddg)
 fa=open(options.ddg,"r")
 for line in fa:
    if line.startswith("#"): continue
    form,ddg=line.split()
    ddg_dict.setdefault(form,float(ddg))
 fa.close()


 ppi_sets=set()
 for input_file in files:
   if input_file.endswith(".ppi"): 
      ppi_sets.add(input_file)

 ddg_real_dict={}
 ddg_pred_dict={}
 comparison={}
 comparison_all={}
 for file_ppi in ppi_sets:
    ppis=[]
    fd=open(os.path.join(input_dir,file_ppi),'r')
    for line in fd:
      pair=line.split()
      ppis.append((pair[0],pair[1]))
    fd.close()
    root_ppi=file_ppi.rstrip(".ppi")
    data=root_ppi.split("_")
    label=data[0]
    wt=(data[1],data[-1])
    ddg_file=root_ppi+"."+score+"_distribution_statistic.out"
    if verbose: print "Check file  %s"%os.path.join(input_dir,ddg_file)
    if fileExist(os.path.join(input_dir,ddg_file)):
      if verbose: print "Open %s"%os.path.join(input_dir,ddg_file)
      fd=open(os.path.join(input_dir,ddg_file),'r')
      for line in fd:
       pair=line.strip().split("versus")
       statistic=line.strip().split()
       if len(statistic)>1:
         if statistic[-1] == "Statistic": statistic_type=statistic[0]
       if len(pair)>1:
        ppia=pair[0].split()[0]
        ppib=pair[1].split()[0]
        pval_averages=pair[1].split("p-value")[1]
        pvalue=pval_averages.split()[0]
        average_preform_a=pval_averages.split()[1]
        average_inform_a=average_preform_a.split("(")[1].split("#")[0]
        average_form_a="::".join(average_inform_a.split("-"))
        average_form_a_r=average_inform_a.split("-")[1]+"::"+average_inform_a.split("-")[0]
        average_value_a=pval_averages.split()[2]
        average_preform_b=pval_averages.split()[3]
        average_inform_b=average_preform_b.split("(")[1].split("#")[0]
        average_form_b="::".join(average_inform_b.split("-"))
        average_form_b_r=average_inform_b.split("-")[1]+"::"+average_inform_b.split("-")[0]
        average_value_b=pval_averages.split()[4]
        p,z=ppia.split("-")
        q,conformer_a= z.split("#")
        cluster_a,interface_a= conformer_a.split("_")
        overlap_a=int(interface_a.lstrip("i"))
        x,z=ppib.split("-")
        y,conformer_b = z.split("#")
        cluster_b,interface_b= conformer_b.split("_")
        overlap_b=int(interface_b.lstrip("i"))
        if int(cluster_a) == int(cluster_b):
          diff = float(average_value_b)-float(average_value_a)
          ddg_a = 0.0
          ddg_b = 0.0
          root_a=average_form_a.split("_")[0]
          mut_a ="_".join([mx for mx in average_form_a.split("_")[1:]])
          root_b=average_form_b.split("_")[0]
          mut_b ="_".join([mx for mx in average_form_b.split("_")[1:]])
          interactors_all=wt[0]+"::"+wt[1]
          if ddg_dict.has_key(average_form_a): ddg_a = ddg_dict.get(average_form_a)
          if ddg_dict.has_key(average_form_a_r): ddg_a = ddg_dict.get(average_form_a_r)
          if ddg_dict.has_key(average_form_b): ddg_b = ddg_dict.get(average_form_b)
          if ddg_dict.has_key(average_form_b_r): ddg_b = ddg_dict.get(average_form_b_r)
          ddg = ddg_b - ddg_a
          comparison_all.setdefault((interactors_all,statistic_type),[]).append((cluster_b,overlap_b,overlap_a,average_form_b,average_form_a,line.strip(),float(pvalue),float(average_value_b),float(average_value_a),diff,ddg))
          if (p,q) == wt or (q,p) == wt :
           if overlap_b > 0:
            if x>y:  interactors= x+"::"+y
            else:    interactors= y+"::"+x
            diff=float(average_value_b)-float(average_value_a)
            if ddg_dict.has_key(average_form_b): ddg=ddg_dict.get(average_form_b)
            if ddg_dict.has_key(average_form_b_r): ddg=ddg_dict.get(average_form_b_r)
            comparison.setdefault((interactors,statistic_type),[]).append((overlap_b,cluster_b,line.strip(),float(pvalue),float(average_value_b),float(average_value_a),diff,ddg))
          if (x,y) == wt or (y,x) == wt  :
           if overlap_a >0:
            if p>q:  interactors= p+"::"+q
            else:    interactors= q+"::"+p 
            diff=float(average_value_a)-float(average_value_b)
            if ddg_dict.has_key(average_form_a): ddg=ddg_dict.get(average_form_a)
            if ddg_dict.has_key(average_form_a_r): ddg=ddg_dict.get(average_form_a_r)
            comparison.setdefault((interactors,statistic_type),[]).append((overlap_a,cluster_a, line.strip(),float(pvalue),float(average_value_a),float(average_value_b),diff,ddg))
      fd.close()

 for (file_name,analysis),result in comparison.iteritems(): 
  fo=open(os.path.join(outdir,"Select_Cluster_with_WT_"+score+"_"+file_name+"_"+analysis+".out"),"w") 
  if verbose: print "Write results in %s"%(os.path.join(outdir,"Select_Cluster_with_WT_"+score+"_"+file_name+"_"+analysis+".out"))     
  for overlap,cluster,data,pvalue,ave_b,ave_a,diff,ddg in sorted(result,key=lambda x: x[0],reverse=True):
    #print "Percentage mutation %s Cluster %s  Data %s \n"%(overlap,cluster,data)
    fo.write("Percentage of mutation in interface around %d %s   Cluster %s  P-value: %f Average-Mutant: %f Average-WT: %f Difference: %f ddG: %f Data %s \n"%(10*int(overlap),"%",cluster,pvalue,ave_b,ave_a,diff,ddg,data))
  fo.close()
         

 for (file_name,analysis),result in comparison_all.iteritems(): 
  fo=open(os.path.join(outdir,"Select_Cluster_all_mutants_"+score+"_"+file_name+"_"+analysis+".out"),"w") 
  if verbose: print "Write results in %s"%(os.path.join(outdir,"Select_Cluster_all_mutants_"+score+"_"+file_name+"_"+analysis+".out"))     
  for cluster,overlap_b,overlap_a,form_b,form_a,data,pvalue,ave_b,ave_a,diff,ddg in sorted(result,key=lambda x: int(x[0]),reverse=True):
    #print "Percentage mutation %s Cluster %s  Data %s \n"%(overlap,cluster,data)
    fo.write("Percentage of mutation in interface around %d %s and %d %s  Cluster %s  P-value: %f Average %s: %f Average %s: %f Difference: %f ddG: %f Data %s \n"%(10*int(overlap_b),"%",10*int(overlap_a),"%",cluster,pvalue,form_b,ave_b,form_a,ave_a,diff,ddg,data))
    if pvalue < p_threshold and int(cluster) <=rank:
        if (10*int(overlap_b) >= percentil or len(form_b.split("_"))==1) and (10*int(overlap_a) >= percentil or len(form_a.split("_"))==1):
            if form_b < form_a:
                ddg_real_dict.setdefault((form_b,form_a),ddg)
                ddg_pred_dict.setdefault((form_b,form_a),diff)
            else:
                ddg_real_dict.setdefault((form_a,form_b),ddg)
                ddg_pred_dict.setdefault((form_a,form_b),diff)
  fo.close()


 ddg_list=[]
 ave_list=[]
 for forms,ene in ddg_real_dict.iteritems():
   if ddg_pred_dict.has_key(forms):
      ddg_list.append(ene)
      ave_list.append(factor*ddg_pred_dict.get(forms))
      

 if len(ave_list)<=0:
     sys.stderr.write("Exit: this set is empty. Please change your conditions\n")
     exit(0)

 ddg_real=np.array(ddg_list)
 ddg_pred=np.array(ave_list)


 (pearson,prob)=st.pearsonr(ddg_real,ddg_pred)

 A = np.vstack([ddg_pred, np.ones(len(ddg_pred))]).T
 m, c = np.linalg.lstsq(A, ddg_real,rcond=-1 )[0]

 output=outdir+"/Compare_ddG_with_average_"+label_out+"_"+score+".dat"
 graph=outdir+"/Compare_ddG_with_average_"+label_out+"_"+score+"."+options.format
 datagraph=outdir+"/Compare_ddG_with_average_"+label+"_"+score+"."+options.format+".dat"

 
 
 if float(options.outlier_percentil) > 0:
   (ddg_list_reduced,ave_list_reduced)=reduce_outliers_lstsq(ddg_list,ave_list,m,c,float(options.outlier_percentil))
   ddg_real=np.array(ddg_list_reduced)
   ddg_pred=np.array(ave_list_reduced)
   (pearson,prob)=st.pearsonr(ddg_real,ddg_pred)
   A = np.vstack([ddg_pred, np.ones(len(ddg_pred))]).T
   m, c = np.linalg.lstsq(A, ddg_real,rcond=-1 )[0]


 fo=open(output,"w")
 fo.write("Pearson Correlation:\t%f\n"%pearson)
 fo.write("Two-tailed p-value: \t%f\n"%prob)
 fo.write("Slope of fitting:   \t%f\n"%m)
 fo.write("Cut Y-axis:         \t%f\n"%c)
 fo.close()

 fo=open(datagraph,"w")
 fo.write("%15s\t%15s\n"%("ddG predict","ddG real"))
 for i in xrange(0,len(ddg_pred)):
  fo.write("%15.5f\t%15.5f\n"%(ddg_pred[i],ddg_real[i]))
 fo.close()



 image, fig = plt.subplots()

 fig.plot(ddg_pred, ddg_real, 'o', label='SKEMPI ddG', markersize=2)
 fig.plot(ddg_pred, m*ddg_pred + c, 'r', label='Fitted line')
 fig.legend()

 fig.set(xlabel='ddG predicted (Kcal/mol)', ylabel='ddG experimental (Kcal/mol)',
       title='Compare ddG of SKEMPI and ddG predicted by MODPIN')
 image.savefig( graph)
 if options.canvas:
    plt.show()


  

# MAIN ###########################################################################################################

if __name__ == '__main__':
    main()   




import sys, os, argparse, gzip

from SBI.beans           import Path
from SBI.databases import SCOPlink

from archdb.src import Source, SCOP
from archdb.src import start_transaction, end_transaction

# User Options
def options(*args, **kwds):
    parser = argparse.ArgumentParser(prog = 'SCOP2SQL',  formatter_class = argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-d', '--database',     dest     = "database", 
                        action = "store",       required = True,  help = "Destination Directory for SCOP database.")
    parser.add_argument('-s', '--sql',          dest     = "sqlfile", 
                        action = "store",       required = True,  help = "Output SQL file")
    parser.add_argument('-k', '--skipdownload', dest     = "skip_download", 
                        action = "store_true",  default  = False,  help = "Skips downloading the database.")
    parser.add_argument('-v', '--verbose',      dest     = "verbose", 
                        action = "store_true",  default  = False, help = "Verbose Mode.")
    options = parser.parse_args()
    return options

def SCOP2SQL(database, sqlfile, skip_download, verbose):
    scop_connect = SCOPlink(local = database)
    newsource      = None
    if not skip_download:
        if verbose: sys.stderr.write("Downloading SCOP database to {0} ...\n".format(database))
        scop_connect.download()
        newsource = Source(name = 'enzyme', source = scop_connect.source)
        if verbose: sys.stderr.write("Download Finished.\n")
    else:
        if verbose: sys.stderr.write("Using previously downloaded database.\n")

    if verbose: sys.stderr.write("Parsing SCOP.\n")
    if verbose: sys.stderr.write("Writing {0} ....\n".format(sqlfile))
    Path.mkdir(os.path.split(os.path.abspath(sqlfile))[0])
    sql_fd = gzip.open(sqlfile, 'wb')
    sql_fd.write(start_transaction())
    if newsource is not None:
        sql_fd.write(newsource.toSQL())

    transfers = []
    scop_obj = SCOP()
    for line in scop_connect.descriptions:
        scop_obj.add_description(line.strip())
    for line in scop_connect.relations:
        scop_obj.add_relation(line.strip())

    sql_fd.write(SCOP.prepdbdeleted())
    sql_fd.write(scop_obj.toSQL())
    sql_fd.write(SCOP.afterpdbdeleted())
    
    sql_fd.write(end_transaction())
    sql_fd.close()
    if verbose: sys.stderr.write("End execution.\n")

if __name__ == "__main__":
    options = options()
    SCOP2SQL(options.database, options.sqlfile, options.skip_download, options.verbose)



import sys
import os
from Bio import PDB
import numpy as np
import scipy as sci
from scipy import spatial
import string
import argparse

def main():
 options = parse_user_arguments()
 try:
  pdbdir,interface,cluster=filter_homologs(options.input,options.potential,options.radius,options.overlap,options.show)
  #Write outputs
  fg=open(options.output+"_list.dat","w")
  for c,s in cluster.iteritems():
   pdbroot  =c.split(".")
   seed     ="_".join([str(x) for x in pdbroot[:-1]])
   seed_list=options.output+"_cluster_"+seed+".dat"
   fd=open(seed_list,"w")
   grouped="\t"
   for p in s:
    pdb=pdbdir+"/"+p
    fd.write("%s\n"%(pdb))
    if p != c:
      z=0.0
      if len(interface[c]) >0:
        z=100 * float(len(interface[c].intersection(interface[p])))/len(interface[c])
      pb=p.split(".")
      grouped += ".".join([str(x) for x in pb[1:-1]])
      grouped += " ({0:5.1f})\t".format(z)
   fd.close()
   fg.write("%s\t%s\n"%(pdbdir+"/"+c,grouped))
  fg.close()
 except IOError as e:
  print("I/O error (%s): %s" %(e.errno, e.strerror))

def parse_user_arguments(*args, **kwds):
 parser = argparse.ArgumentParser(
  description = "Cluster Similar Interactions from a list of PDB binary interactions (pairs)",
  epilog      = "@oliva's lab 2014")
 parser.add_argument('-l','--list_input_file',dest='input',action = 'store',default='input_list',
  help = 'Input file (default is input_list)')
 parser.add_argument('-o','--filtered_output_file',dest='output',action = 'store',default='output',
  help = 'Root-name for output files (default is "output")')
 parser.add_argument('-c','--distance_radius_cutoff',dest='radius',type=float,action = 'store',default='5.0',
  help = 'Distance cut-off to calculate contact residue-pairs of the interface (default 5.0 Ang)')
 parser.add_argument('-pot','--contact_atoms',dest='potential',action = 'store',default='min',
  help = 'Atoms to calculate the distance "min": any closest atom; "cb": CB-CB; "ca": CA-CA')
 parser.add_argument('-id','--percentage_identity',dest='overlap',action = 'store',type=float,default='20.0',
  help = 'Minimum percentage of identical or overlaping pairs of contacts in clusters (default 20.0)')
 parser.add_argument('-v','--verbose',dest='show',action = 'store_true',
  help = 'Verbose execution')

 options=parser.parse_args()
 return options

def fileExist (file):
 return os.path.exists(file) and os.path.isfile(file)

def printverbose(f,flag,message):
 """Define verbose to print in file 'f', if 'flag'=True, message given as 'message'"""
 if flag: f.write("%s"%(message))



def filter_homologs(input_file,potential="min",radius=5.0,overlap=20.0,show=False):

 if fileExist(input_file):
  inp=open(input_file,"r")
  pdblist=[]
  pdbdict={}
  for line in inp:
   word=line.strip().split()
   pdbpath=word[0].split("/")
   pdbfile=pdbpath[-1]
   pdbdir="/".join([str(x) for x in pdbpath[:-1]])
   pdbroot=pdbfile.split(".")
   pdbdict.setdefault(pdbroot[0],[]).append(pdbfile)
   printverbose(sys.stdout,show,("Add in the  list[%s]: %s\n"%(pdbroot[0],pdbfile)))
   pdblist.append(pdbfile)
  inp.close()
 else:
  raise IOError("Input File not found")

 printverbose(sys.stdout,show,("PDB directory: %s\n"%(pdbdir)))

 interface={}
 for pdb in pdblist:
   try:
     pdbfile=pdbdir+"/"+pdb
     printverbose(sys.stdout,show,("PDB file checking: %s\n"%(pdbfile)))
     interface.setdefault(pdb,set()).update(contact(potential,radius,show,pdbfile))
   except Exception as e:
     print("Error: %s" %(e))

 pdbset=set(pdblist)
 cluster={}
 for pa in pdblist:
  if pa in pdbset:
   cluster.setdefault(pa,set()).add(pa)
   printverbose(sys.stdout,show,("Create Cluster centered on: %s\n"%(pa)))
   pdbset.remove(pa)
   for pb in pdblist:
    if pb in pdbset:
     z=0.0
     if len(interface[pa]) >0:
      z=100 * float(len(interface[pa].intersection(interface[pb])))/len(interface[pa])
     if z >= overlap:
      printverbose(sys.stdout,show,("Add in cluster[%s]: %s (id=%10.2f)\n"%(pa,pb,z)))
      cluster[pa].add(pb)
      pdbset.remove(pb)
 return (pdbdir,interface,cluster)

def contact(potential,radius, show,pdbfile):

 if fileExist(pdbfile):
  parser= PDB.PDBParser(QUIET=True)
  fp=open(pdbfile,"r")
  structure=parser.get_structure("complex",fp)
 else:
  raise IOError("File %s not found"%(pdbfile))

 if potential.lower() == "min":
  atoms=[]
  for a in structure.get_atoms():
   atoms.append(a)
  pairs=neighbourhood(atoms,radius,show)
 elif potential.lower() == "cb":
  atoms=[]
  for a in structure.get_atoms():
   r=PDB.Selection.unfold_entities([a], 'R')
   if a.get_name() == 'CB' or (a.get_name() == 'CA' and  r[0].get_resname() =='GLY'):
    atoms.append(a)
  pairs=neighbourhood(atoms,radius,show)
 elif potential.lower() == "ca":
  atoms=[]
  for a in structure.get_atoms():
   if a.get_name() == 'CA' :
    atoms.append(a)
  pairs=neighbourhood(atoms,radius,show)
 else:
  atoms=[]
  for a in structure.get_atoms():
   atoms.append(a)
  pairs=neighbourhood(atoms,radius,show)

 residue_pairs=set()
 for r1,a,r2,b in pairs:
   res1=str(r1.get_id()[1])+"_"+str(r1.get_resname())
   res2=str(r2.get_id()[1])+"_"+str(r2.get_resname())
   residue_pairs.add((res1,res2))
   residue_pairs.add((res2,res1))
 return residue_pairs


def neighbourhood(atoms,radius,show):
 pairs=set()
 for r1,r2 in PDB.NeighborSearch(atoms).search_all(radius,'A'):
   chain_list_1 = PDB.Selection.unfold_entities([r1], 'C')
   chain_list_2 = PDB.Selection.unfold_entities([r2], 'C')
   res_list_1 = PDB.Selection.unfold_entities([r1], 'R')
   res_list_2 = PDB.Selection.unfold_entities([r2], 'R')
   if chain_list_1[0].get_id() != chain_list_2[0].get_id():
      printverbose(sys.stdout,show,(" %s %s (Atom %s) (chain %s)  contact with %s %s (Atom %s) (chain %s) \n"%
            (res_list_1[0].get_resname(),res_list_1[0].get_id()[1],r1.get_id(),chain_list_1[0].get_id(),
             res_list_2[0].get_resname(),res_list_2[0].get_id()[1],r2.get_id(),chain_list_2[0].get_id())))
      pairs.add( ( res_list_1[0], chain_list_1[0].get_id(), res_list_2[0], chain_list_2[0].get_id() ) )
      pairs.add( ( res_list_2[0], chain_list_2[0].get_id(), res_list_1[0], chain_list_1[0].get_id() ) )
 return pairs


if __name__=="__main__":
  main()




from SBI.beans.IndexedNum import IndexedNum
from SBI.beans.JSONer     import JSONer

class SecondaryStructureInfo(JSONer):
    # http://www.wwpdb.org/documentation/format32/sect5.html

    _sstype = '-'

    def __init__(self, line):
        self._line = line

    #
    # ATTRIBUTES
    #
    @property
    def line(self):
        return self._line

    @property
    def chain(self):
        raise NotImplemented

    @property
    def init(self):
        raise NotImplemented

    @property
    def end(self):
        raise NotImplemented

    @property
    def sstype(self):
        return self._sstype

    #
    # FUNCTIONS
    #
    def change_chain(self, chain):
        raise NotImplemented

    def __str__(self):
        return self.line

    #
    # OUT FUNCTIONS
    #
    def as_dict(self):
        nobj = {}
        nobj['type']  = self.sstype
        nobj['ini']   = int(self.init)
        nobj['idxs']  = self.init.index
        nobj['end']   = int(self.end)
        nobj['idxe']  = self.end.index
        nobj['chain'] = self.chain
        nobj['line']  = self.line
        return nobj


class HelixInfo(SecondaryStructureInfo):

    _sstype = 'H'
    # pdb2v7b -> I've been unable to find the correct def for code 0
    pdb_helix_def = {'0' : 'Right-handed alpha',
                     '1' : 'Right-handed alpha',
                     '2' : 'Right-handed omega',
                     '3' : 'Right-handed pi',
                     '4' : 'Right-handed gamma',
                     '5' : 'Right-handed 310',
                     '6' : 'Left-handed alpha',
                     '7' : 'Left-handed omega',
                     '8' : 'Left-handed gamma',
                     '9' : '27 ribbon/helix',
                     '10': 'Polyproline'}

    def __init__(self, line):
        super(HelixInfo, self).__init__(line)
        self._code = line[38:40].strip()

    #
    # ATTRIBUTES
    #
    @property
    def chain(self):
        return self.line[19]

    @property
    def init(self):
        return IndexedNum(self.line[20:27].strip())

    @property
    def end(self):
        return IndexedNum(self.line[32:39].strip())

    @property
    def helix_type(self):
        return self.pdb_helix_def[str(self._code)]

    #
    # FUNCTIONS
    #
    def change_chain(self, chain):
        '''
                 1         2         3         4...7
        1234567890123456789012345678901234567890...0123456
        HELIX    1  HA GLY A   86  GLY A   94  1...      9
        HELIX    2  HB GLY B   86  GLY B   94  1...      9
        '''
        string_line     = list(self.line)
        string_line[19] = chain
        string_line[31] = chain
        self.line       = "".join(string_line)

    def defines(self, chain, pos1, pos2):
        defpos1  = self.line[20:27].strip()
        defpos2  = self.line[32:38].strip()

        defchain = self.line[19:21].strip()
        if chain != defchain:
            return False

        try:
            defpos1int = int(defpos1)
        except:
            defpos1int = int(defpos1[:-1])
        try:
            defpos2int = int(defpos2)
        except:
            defpos2int = int(defpos2[:-1])
        try:
            pos1int = int(pos1)
        except:
            pos1int = int(pos1[:-1])
        try:
            pos2int = int(pos2)
        except:
            pos2int = int(pos2[:-1])

        return (pos1int >= defpos1int and pos1int <= defpos2int) or \
               (pos2int >= defpos1int and pos2int <= defpos2int) or \
               (pos1int <= defpos1int and pos2int >= defpos2int)


class SheetInfo(SecondaryStructureInfo):

    _sstype = 'E'

    def __init__(self, line):
        super(SheetInfo, self).__init__(line)

    #
    # ATTRIBUTES
    #
    @property
    def chain(self):
        return self.line[21]

    @property
    def init(self):
        return IndexedNum(self.line[22:28].strip())

    @property
    def end(self):
        return IndexedNum(self.line[33:38].strip())

    @property
    def beta_direction(self):
        return self.line[38:40].strip()

    def change_chain(self, chain):
        '''
                 1         2         3         4         5         6         7
        1234567890123456789012345678901234567890123456789012345678901234567890
        SHEET    1   A 5 THR A 107  ARG A 110  0
        SHEET    2   A 5 ILE A  96  THR A  99 -1  N  LYS A  98   O  THR A 107
        SHEET    3   A 5 ARG A  87  SER A  91 -1  N  LEU A  89   O  TYR A  97
        SHEET    4   A 5 TRP A  71  ASP A  75 -1  N  ALA A  74   O  ILE A  88
        SHEET    5   A 5 GLY A  52  PHE A  56 -1  N  PHE A  56   O  TRP A  71
        '''
        string_line     = list(self.line)
        string_line[21] = chain
        string_line[32] = chain

        if (len(string_line) > 45 and string_line[49] != ' '):
            string_line[49] = chain
            string_line[64] = chain

        self.line       = "".join(string_line)

    def defines(self, chain, pos1, pos2):
        defpos1 = self.line[22:28].strip()
        defpos2 = self.line[33:38].strip()

        defchain = self.line[21:22].strip()
        if chain != defchain:
            return False

        try:
            defpos1int = int(defpos1)
        except:
            defpos1int = int(defpos1[:-1])
        try:
            defpos2int = int(defpos2)
        except:
            defpos2int = int(defpos2[:-1])
        try:
            pos1int = int(pos1)
        except:
            pos1int = int(pos1[:-1])
        try:
            pos2int = int(pos2)
        except:
            pos2int = int(pos2[:-1])

        decission = (pos1int >= defpos1int and pos1int <= defpos2int) or \
                    (pos2int >= defpos1int and pos2int <= defpos2int) or \
                    (pos1int <= defpos1int and pos2int >= defpos2int)

        double = (len(self.line.strip()) > 45 and self.line[39] == '1')

        if (double and not decission):
            defpos1 = self.line[50:56].strip()
            defpos2 = self.line[65:].strip()
            try:
                defpos1int = int(defpos1)
            except:
                defpos1int = int(defpos1[:-1])
            try:
                defpos2int = int(defpos2)
            except:
                defpos2int = int(defpos2[:-1])

            decission = (pos1int >= defpos1int and pos1int <= defpos2int) or \
                        (pos2int >= defpos1int and pos2int <= defpos2int) or \
                        (pos1int <= defpos1int and pos2int >= defpos2int)

        return decission


class TurnInfo(SecondaryStructureInfo):

    _sstype = 'C'

    def __init__(self, line):
        super(TurnInfo, self).__init__(line)

    #
    # ATTRIBUTES
    #
    @property
    def chain(self):
        return self.line[19]

    @property
    def init(self):
        return IndexedNum(self.line[20:26].strip())

    @property
    def end(self):
        return IndexedNum(self.line[31:39].strip())

    @property
    def turn_type(self):
        return self.line[40:].strip()

    #
    # FUNCTIONS
    #
    def change_chain(self, chain):
        '''
                 1         2         3         4         5
        12345678901234567890123456789012345678901234567890
        TURN     1 S1A GLY A  16  GLN A  18     SURFACE
        TURN     2 FLA ILE A  50  GLY A  52     FLAP
        TURN     3 S2A ILE A  66  HIS A  69     SURFACE
        TURN     4 S1B GLY B  16  GLN B  18     SURFACE
        TURN     5 FLB ILE B  50  GLY B  52     FLAP
        TURN     6 S2B ILE B  66  HIS B  69     SURFACE
        '''
        string_line     = list(self.line)
        string_line[19] = chain
        string_line[30] = chain
        self.line       = "".join(string_line)

    def defines(self, chain, pos1, pos2):
        defpos1 = self.line[22:26].strip()
        defpos2 = self.line[31:37].strip()

        defchain = self.line[19:21].strip()
        if chain != defchain:
            return False

        try:
            defpos1int = int(defpos1)
        except:
            defpos1int = int(defpos1[:-1])
        try:
            defpos2int = int(defpos2)
        except:
            defpos2int = int(defpos2[:-1])
        try:
            pos1int = int(pos1)
        except:
            pos1int = int(pos1[:-1])
        try:
            pos2int = int(pos2)
        except:
            pos2int = int(pos2[:-1])

        return (pos1int >= defpos1int and pos1int <= defpos2int) or \
               (pos2int >= defpos1int and pos2int <= defpos2int) or \
               (pos1int <= defpos1int and pos2int >= defpos2int)



'''
jbonet @ boliva's lab 2013
'''

import os, sys, time, copy, re, ConfigParser

from SBI.beans.Executable import Executable
from SBI.beans.File       import File
from SBI.external.CDhit   import CDhitList  as CHL

global default_configuration_file
default_configuration_file = os.path.join(os.path.normpath(os.path.join(os.path.dirname(__file__),'..')),'configSBI.txt')

class CDhitExe(object):

    def __init__(self, fasta, threshold, output_dir = None, execute = True):
        fasta = os.path.abspath(fasta)
        if output_dir is None:
            output_dir = os.path.split(fasta)[0]

        if   threshold >= 0.7: word = 5
        elif threshold >= 0.6: word = 4
        elif threshold >= 0.5: word = 3
        elif threshold >= 0.4: word = 2
        else:                  word = 1

        #CDhit executable configuration
        self._configurator = ConfigParser.RawConfigParser(allow_no_value=True)
        self._configurator.read(os.getenv('SBI_CONFIG_FILE',default_configuration_file))
        self._exe    = Executable(executable    = self._configurator.get('cd-hit','executable'),
                                  path          = self._configurator.get('cd-hit','path'))

        self._input     = fasta
        output_file     = os.path.split(fasta)[1] + '.' + str(threshold).replace('.','_')
        self._output    = os.path.join(output_dir, output_file)
        self._threshold = str(threshold)
        self._word      = word

        if execute:
            self._execute()

    @property
    def output_file(self):
         return self._output + '.clstr'

    def _execute(self):
        self._exe.add_attribute(self._input,     '-i')
        self._exe.add_attribute(self._output,    '-o')
        self._exe.add_attribute(self._threshold, '-c')
        self._exe.add_attribute('1',             '-g')
        self._exe.add_attribute(self._word,      '-n')

        try:
            self._exe.execute(silent = True)
        except SystemError, e:
            sys.stderr.write('Some error occurred while executing cd-hit\n{0}\n'.format(e))

    def parse(self):
        return CHL.CDhitList(self._output + '.clstr')

    @staticmethod
    def parse_cdhit_output(self, cdhitfile):
        return CHL.CDhitList(cdhitfile)




import sys
import os
import numpy as np
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
from matplotlib.patches import Polygon
import string
import argparse
from scipy import stats

def parse_user_arguments(*args, **kwds):
 parser = argparse.ArgumentParser(
  description = "Create a BOXPLOT with a list of data files",
  epilog      = "@oliva's lab 2014")
 parser.add_argument('-l','--list_input_file',dest='input',action = 'store',default='input_list',
  help = 'Input file (default is input_list)')
 parser.add_argument('-f','--format_image',dest='format',action = 'store',default='png',
  help = 'Format of the image in the output file: png(default), pdf, ps, eps and svg ')
 parser.add_argument('-o','--output_image',dest='out',action = 'store',default='output',
  help = 'Output file name for the imag boxplot')
 parser.add_argument('-top','--Top_Y_value',dest='top',action = 'store',default=0,type=float,
  help = 'Top value of Y-axis')
 parser.add_argument('-bottom','--Botom_Y_value',dest='bottom',action = 'store',default=0,type=float,
  help = 'Bottom value of Y-axis')
 parser.add_argument('-s','--show_canvas',dest='canvas',action = 'store_true',
  help = 'Show canvas figure after execution (not shown by default)')
 parser.add_argument('-g','--show_grid',dest='grid',action = 'store_true',
  help = 'Add horizontal gridlines (default is without)')
 parser.add_argument('-legend','--show_legend',dest='legend',action = 'store_true',
  help = 'Add legends (default is without)')
 parser.add_argument('-t','--title_of_plot',dest='title',action = 'store',default='Boxplot',
  help = 'Title of the plot within "" ')
 parser.add_argument('-nc','--set_colored_groups',dest='color',action = 'store',type=int, default=1,
  help = 'Number of sets grouped with the same color. Up to 14 different colors (default 1 => every boxplot has different color)')
 parser.add_argument('-nh','--set_hashed_groups',dest='hashed',action = 'store',type=int, default=1,
  help = 'Number of sets grouped with the same filling hash. Up to 10 different filling hashes (default 1 => every boxplot has different hash)')
 parser.add_argument('-v','--verbose',dest='show',action = 'store_true',
  help = 'Verbose execution')

 options=parser.parse_args()
 return options

def fileExist (file):
 return os.path.exists(file) and os.path.isfile(file)

def printverbose(f,flag,message):
 """Define verbose to print in file 'f', if 'flag'=True, message given as 'message'"""
 if flag: f.write("%s"%(message))

def main():
 options = parse_user_arguments()
 try:
  boxplotlist(options)
 except IOError as e:
  print("I/O error (%s): %s" %(e.errno, e.strerror))

def boxplotlist(options):

 if fileExist(options.input):
  dataname=[]
  dataplot=[]
  fp=open(options.input,"r")
  for line in fp:
   (name,datafile)=line.strip().split()
   if fileExist(datafile):
     dataname.append(name)
     datavalue=[]
     fi=open(datafile,"r")
     printverbose(sys.stdout,options.show,"Open %s\n"%(datafile))
     for datainfo in fi:
       (info,value)=datainfo.strip().split()
       datavalue.append(float(value))
     dataplot.append(np.array(datavalue))
     fi.close()
  fp.close()
 else:
  raise IOError("Input File not found")

 fig, ax1 = plt.subplots(figsize=(10,6))
 title='Boxplot of '+ options.input
 fig.canvas.set_window_title(title)
 plt.subplots_adjust(left=0.075, right=0.95, top=0.9, bottom=0.25)

 printverbose(sys.stdout,options.show,"Data %s %s\n"%(repr(dataname),repr(dataplot)))
 bp = plt.boxplot(dataplot, notch=0, sym='+', vert=1, whis=1.5)
 plt.setp(bp['boxes'], color='black')
 plt.setp(bp['whiskers'], color='black')
 plt.setp(bp['fliers'], color='red', marker='+')

# Add a horizontal grid to the plot, but make it very light in color
# so we can use it for reading data values but not be distracting
 if options.grid:
   ax1.yaxis.grid(True, linestyle='-', which='major', color='lightgrey',
              alpha=0.5)

# Hide these grid behind plot objects
 ax1.set_axisbelow(True)

# Add Title of the plot
 ax1.set_title(options.title)

 ax1.set_xlabel('Samples')
 ax1.set_ylabel('Value')

# Fill the boxes with desired colors and hashes
 boxColors = ['white','darkkhaki','royalblue','salmon','lime','red','violet','cyan','blue',
             'yellow','green','navy','grey','orange','firebrick']
 boxHatch  = [' ','/','\\','|','-','+','x','o','.','*']

 if options.color==0: options.color=1
 if options.hashed==0: options.hashed=1
 numBoxes = len(dataplot)
 medians = range(numBoxes)
 k=0
 kk=0
# Statistics: compare all vs all data
 ks={}
 mw={} 
 for i in range(numBoxes):
  for j in range(numBoxes):
   try:
    ks.setdefault((i,j),stats.ks_2samp(dataplot[i],dataplot[j]))
   except:
    ks.setdefault((i,j),(0.0,1.0))
    e = sys.exc_info()[0]
    sys.stdout.write("Fail Kolmogorov Smirnov Test %s\n"%repr(e))
   try:
    mw.setdefault((i,j),stats.mannwhitneyu(dataplot[i],dataplot[j]))
   except:
    mw.setdefault((i,j),(0.0,0.5))
    e = sys.exc_info()[0]
    sys.stdout.write("Fail Mann-Whitney Test %s\n"%e)

#Boxplot
 for i in range(numBoxes):
   box = bp['boxes'][i]
   boxX = []
   boxY = []
   for j in range(5):
      boxX.append(box.get_xdata()[j])
      boxY.append(box.get_ydata()[j])
   boxCoords = zip(boxX,boxY)
  # Group by color and hashed lines
   n= k * options.color + i % options.color
   if n!=i:
    k=k+1
    if k>14: k=0
    kk=0
   j= i - k * options.color
   m= kk * options.hashed + j %  options.hashed
   if m!=j:
    kk=kk+1
    if kk>9: kk=0
   boxPolygon = Polygon(boxCoords, facecolor=boxColors[k], hatch= boxHatch[kk])
   ax1.add_patch(boxPolygon)
  # Now draw the median lines back over what we just filled in
   med = bp['medians'][i]
   medianX = []
   medianY = []
   for j in range(2):
       medianX.append(med.get_xdata()[j])
       medianY.append(med.get_ydata()[j])
       plt.plot(medianX, medianY, 'black',linewidth=2.5)
       medians[i] = medianY[0]
  # Finally, overplot the sample averages, with horizontal alignment
  # in the center of each box
   plt.plot([np.average(med.get_xdata())], [np.average(dataplot[i])],
           color='white', marker='*', markeredgecolor='black')

# Set the axes ranges and axes labelsa
 ax1.set_xlim(0.5, numBoxes+0.5)
 if options.top ==0:
  top = max([ max([x for x in dataplot[i]]) for i in range(numBoxes) ])
 else:
  top = options.top
 if options.bottom==0:
  bottom = min([ min([x for x in dataplot[i]]) for i in range(numBoxes) ])
 else:
  bottom=options.bottom
 ax1.set_ylim(bottom, top)
 xtickNames = plt.setp(ax1, xticklabels=dataname)
 plt.setp(xtickNames, rotation=45, fontsize=12)

# Finally, add a basic legend
 if options.legend:
  posy=0.12
  posx=0.7
  k=0
  for i in range(numBoxes):
   # Group by color and hashed lines
    n= k * options.color + i % options.color
    if n!=i:
     k=k+1
    plt.figtext(posx, posy,  dataname[i] ,
            backgroundcolor=boxColors[k], color='black', weight='roman',
            size='x-small')
    posy=posy-0.03
    if posy<0.01:
     posy=0.12
     posx=posx+0.12

  plt.figtext(0.14, 0.1, '*', color='white', backgroundcolor='silver',
           weight='roman', size='medium')
  plt.figtext(0.15, 0.095, ' Average Value', color='black', weight='roman',
           size='x-small')

 output=options.out+"."+options.format
 printverbose(sys.stdout,options.show,"output is %s\n"%output)
 plt.savefig(output)

 sys.stdout.write("Kolmogorov-Smirnov Statistic\n") 
 sys.stdout.write("+--------------------------+\n") 
 for i in range(numBoxes):
  for j in range(numBoxes):
    if j>=i :
     sys.stdout.write("%s versus %s : Statistic %f p-value %f Average(%s)= %f Average(%s)= %f\n"%(dataname[i],dataname[j],ks[(i,j)][0],ks[(i,j)][1],dataname[i],np.average(dataplot[i]),dataname[j],np.average(dataplot[j])))

 sys.stdout.write("\nMann-Whitney U Statistic\n") 
 sys.stdout.write("+----------------------+\n") 
 for i in range(numBoxes):
  for j in range(numBoxes):
    if j>=i :
     sys.stdout.write("%s versus %s : Statistic %f p-value %f Average(%s)= %f Average(%s)= %f\n"%(dataname[i],dataname[j],mw[(i,j)][0],2*mw[(i,j)][1],dataname[i],np.average(dataplot[i]),dataname[j],np.average(dataplot[j])))

     
 if options.canvas:
  mpl.use('Agg')
  plt.show()


if __name__=="__main__":
 main()






'''
@file: Executable.py

@author: Jaume Bonet
@mail:   jaume.bonet@gmail.com
@date:   03/2013

@ [oliva's lab](http://sbi.imim.es)

@class: Executable
@class: ExecutableError
'''

import os
import subprocess
import copy

from .. import SBIglobals as SBIg


class Executable(object):
    '''
    Manages executing external programs through the SBI library.
    '''
    def __init__(self, executable, path=None):
        '''
        @param:    executable
        @pdef:     name of the executable program.
        @ptype:    {String}

        @param:    path
        @pdef:     path to the executable. Not needed if the executable is in
                   the $PATH environment variable.
        @pdefault: _None_
        @ptype:    {String}
        '''
        self._exec = executable
        self._path = path

        self._error = ExecutableError(self.executable)

        if path is not None:
            self._path = os.path.abspath(path)
        else:
            found = self._load_executable_path()
            if not found:
                raise self._error.not_in_path()

        self._error = ExecutableError(self.full_executable)

        self._check_executable()

        self._command = []
        self._command.append(self.full_executable)

        self._backup_command = []

    ##############
    # ATTRIBUTES #
    ##############
    @property
    def executable(self):
        '''
        Name of the executable.

        @return: {String}
        '''
        return self._exec

    @property
    def path(self):
        '''
        Path to the executable.

        @return: {String}
        '''
        return self._path

    @property
    def command(self):
        '''
        Full command to execute.

        @return {List}
        '''
        return self._command

    @property
    def full_executable(self):
        '''
        Full executable with path.

        @return: {String}
        '''
        return os.path.join(self._path, self._exec)

    ###########
    # METHODS #
    ###########
    def add_attribute(self, attribute_value, attribute_id=None):
        '''
        Adds a new parameter to the command.
        Specifically for parameters with 'tags' like '-i'.

        @param:    attribute_value
        @pdef:     value of the attribute to add.
        @ptype:    {String}. Transformed otherwise.

        @param:    attribute_id
        @pdef:     label of the attribute to add.
        @pdefault: _None_
        @ptype:    {String}
        '''
        if attribute_id is None:
            self.add_parameter(attribute_value)
        else:
            self._command.append(attribute_id)
            self._command.append(str(attribute_value))

    def add_parameter(self, parameter):
        '''
        Adds a new stand alone parameter to the command.

        @param:    parameter
        @pdef:     value of the parameter to add.
        @ptype:    {String}. Transformed otherwise.
        '''
        self._command.append(str(parameter))

    def clean_command(self):
        '''
        Removes all attributes and parameters added to the command.
        '''
        self._command = []
        self._command.append(self.full_executable)

    def backup_command(self):
        '''
        Store a copy of the command up to that point to retrieve import
        afterwards.
        '''
        self._backup_command = copy.deepcopy(self._command)

    def restore_command(self):
        '''
        Retrieve the backup command into the working command.
        '''
        self._command        = self._backup_command
        self._backup_command = []

    def execute(self, silent=False):
        '''
        Executes the commands.

        @param:    silent
        @pdef:     External program STDERR is shown through STDERR if _True_
        @pdefault: _False_
        @ptype:    {Boolean}

        @raises: {SystemError} if an error occurs in the external program.
        '''
        outPIPE = subprocess.PIPE if SBIg.debug else open('/dev/null', 'w')
        errPIPE = open('/dev/null', 'w') if silent else subprocess.PIPE

        command = " ".join(self.command)
        SBIg.alert('debug', self, '\tExecuting:\t{0}'.format(command))

        try:
            p = subprocess.Popen(self.command, stdout = outPIPE,
                                               stderr = errPIPE)

            if SBIg.debug:
                for out in iter(p.stdout.readline, b''):
                    SBIg.alert('debug', self, out.strip())
            p.communicate()
        except:
            raise SystemError()

    ###################
    # PRIVATE METHODS #
    ###################
    def _load_executable_path(self):
        '''
        Retrieves the executable path in case self._path is None.

        @return: {Boolean}. _True_ if the path is found, _False_ otherwise.
        '''
        search = ["which", self.executable]
        p = subprocess.Popen(search, stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)
        out, err = p.communicate()

        if out != '':
            self._path = os.path.split(out.strip())[0]
            return True
        else:
            return False

    def _check_executable(self):
        '''
        Checks that the final executable can be executed.

        @raises: {ExecutableError} otherwise.
        '''
        if not os.path.isfile(self.full_executable):
            raise self._error.no_exists()
        if not os.access(self.full_executable, os.X_OK):
            raise self._error.no_permission()

    #################
    # CLASS METHODS #
    #################
    def __repr__(self):
        return " ".join(self._command)

    def __str__(self):
        return repr(self)


class ExecutableError(Exception):
    '''
    Manages different error produced by the {Executable} object.

    '''
    def __init__(self, executable):
        '''
        @param: executable
        @pdef:  full path of the executable.
        @ptype: {String}
        '''
        self._exec = executable
        self._msg  = ''

    ###########
    # METHODS #
    ###########
    def not_in_path(self):
        self._msg = '{0} can not be found. Please specify.'.format(self._exec)
        return self

    def no_exists(self):
        self._msg = '{0} does not exist.'.format(self._exec)
        return self

    def no_permission(self):
        self._msg = 'No permission to execute {0}.'.format(self._exec)
        return self

    #################
    # CLASS METHODS #
    #################
    def __str__(self):
        return self._msg



"""
NucleotideAtom

author: jbonet
date:   02/2013

@oliva's lab
"""

from . import Atom


class AtomOfNucleotide(Atom):
    """
    An {AtomOfNucleotide} is simply a point in space defined by 3 coordinates
    WITH specific functions for atoms in amino acids
    """

    backbone_atoms = set(["P", "OP1", "OP2", "OP3", "O5'", "C5'", "C4'", "O4'", "C3'", "O3'", "C2'", "O2'", "C1'",
                                                    "O5*", "C5*", "C4*", "O4*", "C3*", "O3*", "C2*", "O2*", "C1*"])
    #
    # BOOLEANS
    #

    @property
    def is_Phosphate(self):     return self._name == "P"

    @property
    def is_PhosphoOxygen(self): return self._name in set(["OP1", "OP2", "OP3"])



# from SBI.structure.protein import SSpair
import re
tables = {'chain' : 'chain',
          'ld'    : 'loop_description',
          'sld'   : 'superloop_description',
          'l2c'   : 'loop2chain',
          's2c'   : 'superloop2chain',
          'l2s'   : 'loop2superloop'}


class Arch(object):

    def __init__(self, pdb):
        d = pdb.split('_')
        self._pdb   = d[0]
        self._chain = d[1]
        self._archs = []

    @property
    def archs(self):
        return self._archs

    @archs.setter
    def archs(self, value):
        self._archs.append(value)

    def toSQL(self):
        command = "SET @chain = (SELECT nid FROM {0} WHERE pdb='{1}' AND chain='{2}');\n".format(tables['chain'], self._pdb, self._chain)

        singles = {}
        supers  = {}
        for arch in self._archs:
            # command += str(arch)
            l1  = arch.length_ss1
            l2  = arch.length_ss2
            mi1 = arch._ss1.get_moment_of_inertia_length('f11')
            mi2 = arch._ss2.get_moment_of_inertia_length('f44')
            sec = arch.aminoacid_sequence
            exp = arch.access_surface
            ss  = arch.structure_sequence
            i   = re.search('(-*\d+)(\w*)', arch.initial_position)
            e   = re.search('(-*\d+)(\w*)', arch.end_position)
            ii, ix, ee, ex = i.group(1), ' ' if i.group(2) == '' else i.group(2), e.group(1), ' ' if e.group(2) == '' else e.group(2)
            if not arch.is_superarch:
                table  = tables['ld']
                values = "(loop_id,type,length,ss1L,ss1moiL,ss2L,ss2moiL,distance,theta,rho,delta,sequence,ss,exposition)"
                close  = ");\n"
                archid = arch._source + '_' + str(arch.initial_position)
                singles[int(ii)] = archid.strip()
            else:
                table  = tables['sld']
                values = "(loop_id,type,length,ss1L,ss1moiL,ss2L,ss2moiL,distance,theta,rho,delta,sequence,ss,exposition,internal_ss_count,internal_ss_desc)"
                close  = ",{0._intss},'{0._inttyp}');\n".format(arch)
                archid = arch._source + '_' + arch.initial_position + '_' + str(arch._intss)
                supers[str(ii) + '_' + str(arch._intss)] = archid.strip()
            command += "INSERT INTO {0} {1} VALUES ('{11}','{2.type}',{2._distAA},{4},{5},{6},{7},{2._dist:.3f},{2._angls[0]:.3f},{2._angls[1]:.3f},{2._angls[2]:.3f},'{8}','{9}','{10}'{3}".format(table, values, arch, close, l1, mi1, l2, mi2, sec, ss, exp, archid.strip())
            command += "SET @arch = LAST_INSERT_ID();\n"

            if not arch.is_superarch:
                command += "CALL new_loop2chain(@arch,@chain,{2},'{3}',{4},'{5}','D',{6});\n".format(self._pdb, self._chain, ii, ix, ee, ex, arch._order)
            else:
                command += "CALL new_superloop2chain(@arch,@chain,{2},'{3}',{4},'{5}','D',{6});\n".format(self._pdb, self._chain, ii, ix, ee, ex, arch._order)

        for s in sorted(supers):
            info = s.split('_')
            info[0] = int(re.search('(-*\d+)(\w*)', info[0]).group(1))
            info[1] = int(info[1])
            command += "SET @super = (SELECT nid FROM {0} WHERE loop_id='{1}');\n".format(tables['sld'], supers[s])
            toprint = False
            for a in sorted(singles):
                if info[1] < 0:
                    break
                if a == info[0]:
                    toprint = True
                if toprint:
                    command += "SET @single = (SELECT nid FROM {0} WHERE loop_id='{1}');\n".format(tables['ld'], singles[a])
                    command += "INSERT INTO {0} VALUES (@super,@single);\n".format(tables['l2s'])
                    info[1] -= 1
        return command



import sys, os, argparse

from SBI            import SBIglobals
from SBI.beans      import File, Path
from SBI.databases  import PDBlink
from SBI.utilities  import archer

# User Options
def options(*args, **kwds):
    parser = argparse.ArgumentParser(prog = 'builtArchs',  formatter_class = argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-l', '--list',         dest     = "list", 
                        action = "store",       required = True,  help = "List of selected PDB_chain.")
    parser.add_argument('-p', '--pdbdir',       dest     = "pdbdir", 
                        action = "store",       required = True,  help = "Dir of local PDB database.")
    parser.add_argument('-o', '--output',       dest     = "output", 
                        action = "store",       required = True,  help = "Dir to output the archdata.")
    parser.add_argument('-s', '--sort',         dest     = "sort", 
                        action = "store_true",  default  = False, help = "Skip building loops, only sort them.")
    parser.add_argument('-v', '--verbose',      dest     = "verbose", 
                        action = "store_true",  default  = False, help = "Verbose Mode.")
    options = parser.parse_args()
    return options

def PDBlist2dict(inputfile, separator = '_'):
    data = []
    inputf = File(inputfile,'r')
    for line in inputf.descriptor:
        data.append(line.strip().split(separator))
    return data

def builtArchDB(pdblist, pdbdir, outdir):
    pdb_connect = PDBlink(local = pdbdir)
    for pdbinfo in pdblist:
        pdb, chain  = pdbinfo
        subdir      = pdb[1:3].lower()
        pdbfile     = pdb_connect.get_PDB(pdb)
        SBIglobals.alert('verbose', None, 'Processing file: {0}'.format(pdbfile))
        archs       = archer.build_archs(sourcepdb = pdbfile, chain = chain, limit_distance = 25)
        for archkey in archs:
            if len(archs[archkey]) > 0: 
                Path.mkdir(os.path.join(outdir[archkey],subdir))
                Path.mkdir(os.path.join(outdir['STRUC'],subdir))
            for arch in archs[archkey]:
                pyobjName = "_".join([str(arch.aminoacid_distance), arch.type, arch.identifier]) + '.archObj'
                arch.dump(os.path.join(os.path.join(outdir[archkey],subdir),pyobjName))
                arch.format2file(filename  = os.path.join(os.path.join(outdir['STRUC'],subdir),arch.identifier), 
                                 extension = 'pdb', center = True)
                arch.format2file(filename  = os.path.join(os.path.join(outdir['STRUC'],subdir),arch.identifier), 
                                 extension = 'js',  center = True)

if __name__ == "__main__":
    options            = options()
    SBIglobals.verbose = options.verbose
    pdblist            = PDBlist2dict(options.list)
    outdir = {'ARCHS' : os.path.join(options.output,'archobj'), 
              'SUPER' : os.path.join(options.output,'superobj'),
              'STRUC' : os.path.join(options.output,'structures'),
              'DBARC' : os.path.join(options.output,'db'),
              'DBSUP' : os.path.join(options.output,'superdb')}
    if not options.sort:
        builtArchDB(pdblist, options.pdbdir, outdir)
    archer.sortarchs(outdir['ARCHS'],  outdir['DBARC'])
    archer.sortarchs(outdir['SUPER'],  outdir['DBSUP'])



from setuptools import setup, find_packages

setup(
    name="modpin",  
   	version="1.1.1", 
	author="GRIB",
	author_email="", 
    description="SET OF PYTHON SCRIPTS TO MODEL AND ANALYZE PROTEIN-PROTEIN INTERACTIONS",
	keywords="protein interaction modelling",
	url="",
    classifiers=[  
        "Development Status :: 3 - Alpha",
    ],

    packages=find_packages(),
    include_package_data = True,
	install_requires=[
   		'numpy',
   		'biopython',
		'scipy',
		'matplotlib'
	],
 	
	entry_points = {
        'console_scripts': ['configuration=modpin.config_setup:main',
'modppi=modpin.scripts.modppi:main',
'analysis=modpin.scripts.analysis:main',
'modelist=modpin.scripts.modelist:main',
'FilterHomologPairs=modpin.scripts.FilterHomologPairs:main',
'Renumerate=modpin.scripts.Renumerate:main',
'3did = modpin.src.functions.3did:main',

],
	}

	

)





